var MRAID_ENV = window.MRAID_ENV;
var vrtcal = window.vrtcal;
var mraid = window.mraid;

(function() {
    // Determine if ad is served as MRAID ad
    if (typeof(VrtcalMraidJsBridge) === "undefined") {
        return;
    }

    // Populate MRAID_ENV
    if (typeof(MRAID_ENV) === "undefined") {
        MRAID_ENV = {
            version: "3.0",
            sdk: "VRTCAL",
            sdkVersion: VrtcalMraidJsBridge.getSdkVersion(),
            appId: VrtcalMraidJsBridge.getAppId()
        };

        var limitAdTracking = VrtcalMraidJsBridge.getLimitAdTracking();
        if (typeof(limitAdTracking) !== "undefined") {
            MRAID_ENV.limitAdTracking = limitAdTracking;
        }

        var ifa = VrtcalMraidJsBridge.getAdid();
        if (typeof(ifa) !== "undefined" && limitAdTracking === false) {
            MRAID_ENV.ifa = ifa;
        }

        var coppa = VrtcalMraidJsBridge.getCoppa();
        if (typeof(coppa) !== "undefined") {
            MRAID_ENV.coppa = (coppa === "true");
        }
    }

    if (typeof(vrtcal) === "undefined") {
        vrtcal = {
            state: "loading",
            isTwopartExpandable: true,
            eventListenerMap: {}
        };
    }

    // Provide a basic mraid object, with only ability to query state and add/remove event listeners
    if (typeof(mraid) === "undefined") {
        mraid = {
            getState: function() {
                return vrtcal.state;
            },

            addEventListener: function(event, listener) {
                if (typeof(vrtcal.eventListenerMap[event]) === "undefined") {
                    vrtcal.eventListenerMap[event] = [];
                }
                vrtcal.eventListenerMap[event].push(listener);
            },

            removeEventListener: function(event, listener) {
                if (typeof(listener) === "undefined") {
                    // MRAID specs, section 5.3
                    vrtcal.eventListenerMap[event] = [];
                } else {
                    var index = vrtcal.eventListenerMap[event].indexOf(listener);
                    if (index > -1) {
                        vrtcal.eventListenerMap[event].splice(index);
                    }
                }
            }
        };
    }
})();


(function() {
    vrtcal.DELAY = 0;
    vrtcal.adType = "";
    vrtcal.expandProperties = {};
    vrtcal.orientationProperties = {};
    vrtcal.resizeProperties = {};
    vrtcal.isAdShown = false;
    vrtcal.currentExposure = {x: 0, y: 0, width: 0, height: 0};
    vrtcal.androidLayoutRules = {};
    vrtcal.useCustomClose = false;
    vrtcal.defaultPosition = {x: 0, y: 0, width: 0, height: 0};
    vrtcal.previousSize = {width: 0, height: 0};
    vrtcal.lastClickTime = 0;

    addEventListener("click", function() {
        vrtcal.lastClickTime = (new Date()).getTime();
    }, true);

    vrtcal.isEmpty = function(value) {
        return (typeof(value) === "undefined" || value === null);
    };

    vrtcal.expandAd = function(url) {
        var expandProperties = mraid.getExpandProperties();
        var width = expandProperties.width;
        var height = expandProperties.height;

        vrtcal.log("Invoking expandAd() on SDK with url=" + url + ", width=" + width + ", height=" + height
                + ", useCustomClose=" + vrtcal.useCustomClose
                + ", allowOrientationChange=" + vrtcal.orientationProperties.allowOrientationChange
                + ", forceOrientation=" + vrtcal.orientationProperties.forceOrientation);

        var expandUrl = (typeof(url) === "undefined" || url === null) ? "" : url;
        VrtcalMraidJsBridge.expandAd(expandUrl, width, height, vrtcal.useCustomClose,
                vrtcal.orientationProperties.allowOrientationChange,
                vrtcal.orientationProperties.forceOrientation);
    };

    vrtcal.fireEvent = function(event, param1, param2) {
        var log = "Firing event " + event + "(";
        if (!vrtcal.isEmpty(param1)) {
            if (typeof(param1) === "object") {
                log += JSON.stringify(param1);
            } else if (typeof(param1) === "string") {
                log += "\"" + param1 + "\"";
            } else {
                log += param1;
            }
        }
        if (!vrtcal.isEmpty(param2)) {
            log += ", ";
            if ((typeof param2) === "object") {
                log += JSON.stringify(param2);
            } else if ((typeof param2) === "string") {
                log += "\"" + param2 + "\"";
            } else {
                log += param2;
            }
        }
        log += ")";
        vrtcal.log(log);
        var listenerList = vrtcal.eventListenerMap[event];
        if (!vrtcal.isEmpty(listenerList)) {
            listenerList.forEach(function(listener) {
                listener(param1, param2);
            });
        }
    };

    vrtcal.fireErrorEvent = function(message, action) {
        vrtcal.fireEvent("error", message, action);
    };

    vrtcal.getCurrentPosition = function() {
        return JSON.parse(VrtcalMraidJsBridge.getCurrentPosition());
    };

    vrtcal.getMaxSize = function() {
        return JSON.parse(VrtcalMraidJsBridge.getMaxSize());
    };

    vrtcal.getScreenSize = function() {
        return JSON.parse(VrtcalMraidJsBridge.getScreenSize());
    };

    vrtcal.hasRecentUserInteraction = function() {
        var duration = (new Date()).getTime() - vrtcal.lastClickTime;
        return (duration < 1000) || VrtcalMraidJsBridge.hasRecentUserInteraction();
    }

    vrtcal.log = function(message) {
        console.log("VRTCAL bridge: " + message);
    };

    vrtcal.resizeAd = function() {
        var props = vrtcal.validateResizeProperties(vrtcal.resizeProperties);
        if (!vrtcal.isEmpty(props.error)) {
            vrtcal.fireErrorEvent("Cannot resize ad because resizeProperties validation failed: " + props.error, "resizeAd");
            return;
        }

        vrtcal.log("Invoking resizeAd() on SDK with width=" + props.width + ", height=" + props.height
                + ", x=" + props.x + ", y=" + props.y + ", hLayoutRule=" + props.hLayoutRule + ", vLayoutRule=" + props.vLayoutRule);
        VrtcalMraidJsBridge.resizeAd(props.x, props.y, props.width, props.height, props.hLayoutRule, props.vLayoutRule);
    };

    vrtcal.formatCalendarEventRecurrence = function(recurrence) {
        var result = "";

        if (!vrtcal.isEmpty(recurrence.frequency)) {
            switch (recurrence.frequency) {
                case "daily":
                    result += "FREQ=DAILY;";
                    break;
                case "weekly":
                    result += "FREQ=WEEKLY;";
                    break;
                case "monthly":
                    result += "FREQ=MONTHLY;";
                    break;
                case "yearly":
                    result += "FREQ=YEARLY;";
                    break;
                default:
                    break;
            }
        }

        if (!vrtcal.isEmpty(recurrence.daysInWeek)) {
            var daysOfWeek = ["SU", "MO", "TU", "WE", "TH", "FR", "SA", "SU"];
            var daysInWeek = recurrence.daysInWeek;
            daysInWeek.forEach(function(item, index, array) {
                array[index] = daysOfWeek[item];
            });
            result += "BYDAY=" + daysInWeek.join(",") + ";";
        }
        if (!vrtcal.isEmpty(recurrence.daysInMonth)) {
            result += "BYMONTHDAY=" + recurrence.daysInMonth.join(",") + ";";
        }
        if (!vrtcal.isEmpty(recurrence.monthsInYear)) {
            result += "BYMONTH=" + recurrence.monthsInYear.join(",") + ";";
        }
        if (!vrtcal.isEmpty(recurrence.daysInYear)) {
            result += "BYYEARDAY=" + recurrence.daysInYear.join(",") + ";";
        }

        if (!vrtcal.isEmpty(recurrence.expires)) {
            var date = new Date(recurrence.expires);
            var month = date.getMonth() + 1;
            result += "UNTIL=";
            result += date.getFullYear();
            result += (month < 10) ? ("0" + month) : month;
            result += (date.getDate() < 10) ? ("0" + date.getDate()) : date.getDate();
            result += "T";
            result += (date.getHours() < 10) ? ("0" + date.getHours()) : date.getHours();
            result += (date.getMinutes() < 10) ? ("0" + date.getMinutes()) : date.getMinutes();
            result += (date.getSeconds() < 10) ? ("0" + date.getSeconds()) : date.getSeconds();
            result += "Z;";
        }

        if (result.endsWith(";")) {
            result = result.substring(0, result.length - 1);
        }

        return result;
    };

    vrtcal.validateResizeProperties = function(resizeProperties) {
        vrtcal.log("Checking resizeProperties " + JSON.stringify(resizeProperties));

        if (vrtcal.isEmpty(resizeProperties)) {
            return {error: "resizeProperties is undefined or is null"};
        }
        if (typeof(resizeProperties.width) !== "number") {
            return {error: "Invalid value for width"};
        }
        if (typeof(resizeProperties.height) !== "number") {
            return {error: "Invalid value for height"};
        }
        if (typeof(resizeProperties.offsetX) !== "number") {
            return {error: "Invalid value for offsetX"};
        }
        if (typeof(resizeProperties.offsetY) !== "number") {
            return {error: "Invalid value for offsetY"};
        }

        if (!vrtcal.isEmpty()
                && typeof(resizeProperties.allowOffscreen) !== "boolean") {
            return {error: "Invalid value for allowOffscreen"};
        }

        var width = resizeProperties.width;
        var height = resizeProperties.height;
        var offsetX = resizeProperties.offsetX;
        var offsetY = resizeProperties.offsetY;
        var allowOffscreen = (typeof resizeProperties.allowOffscreen === "undefined") ? true : resizeProperties.allowOffscreen;
        var customClosePosition = (typeof resizeProperties.customClosePosition === "undefined")
            ? "top-right" : resizeProperties.customClosePosition;

        var currentPosition = mraid.getCurrentPosition();
        var x = currentPosition.x + offsetX;
        var y = currentPosition.y + offsetY;

        var maxSize = mraid.getMaxSize();
        var maxWidth = maxSize.width;
        var maxHeight = maxSize.height;

        // Make sure ad will be bigger than close event region
        if (width < 50 || height < 50) {
            return {error: "Ad size is smaller than close event region"};
        }

        if (allowOffscreen === false) {
            // Verify that entire ad fits inside maxSize
            if (width > maxWidth || height > maxHeight) {
                return {error: "Ad size exceeds max size of " + maxSize.width + "," + maxSize.height + " (allowOffscreen is false)"};
            }

            // If parts of ad are outside of maxSize, shift x/y accordingly
            if (x < 0) {
                vrtcal.log("Requested x is less than 0.  Setting x to 0.");
                x = 0;
            }
            if ((x + width) > maxWidth) {
                x = maxWidth - width;
                vrtcal.log("Requested (x + width) is outside of max width.  Moving x to " + x);
            }

            if (y < 0) {
                vrtcal.log("Requested y is less than 0.  Setting y to 0.");
                y = 0;
            }
            if ((y + height) > maxHeight) {
                y = maxHeight - height;
                vrtcal.log("Requested (y + height) is outside of max height.  Moving y to " + y);
            }
        }

        // Verify close region will not be offscreen
        var closeRegionLeft = x + ((width - 50) / 2);
        if (customClosePosition.indexOf("left") !== -1) {
            closeRegionLeft = x;
        } else if (customClosePosition.indexOf("right") !== -1) {
            closeRegionLeft = x + width - 50;
        }

        var closeRegionTop = y + ((height - 50) / 2);
        if (customClosePosition.indexOf("top") !== -1) {
            closeRegionTop = y;
        } else if (customClosePosition.indexOf("bottom") !== -1) {
            closeRegionTop = y + height - 50;
        }

        var closeRegionRight = closeRegionLeft + 50;
        var closeRegionBottom = closeRegionTop + 50;

        if (closeRegionLeft < 0
                || closeRegionTop < 0
                || closeRegionRight > maxWidth
                || closeRegionBottom > maxHeight) {
            return {error: "Close region will not be fully visible ("
                    + "currentPosition=" + JSON.stringify(currentPosition) + ", "
                    + "maxSize=" + JSON.stringify(maxSize) + ", "
                    + "closeRegionLeft=" + closeRegionLeft + ", closeRegionTop=" + closeRegionTop + ", "
                    + "closeRegionRight=" + closeRegionRight + ", closeRegionBottom=" + closeRegionBottom + ", "
                    + ")"};
        }

        var hLayoutRule = vrtcal.androidLayoutRules.CENTER_HORIZONTAL;
        var vLayoutRule = vrtcal.androidLayoutRules.CENTER_VERTICAL;
        if (customClosePosition === "center") {
            hLayoutRule = vrtcal.androidLayoutRules.CENTER_IN_PARENT;
            vLayoutRule = vrtcal.androidLayoutRules.CENTER_IN_PARENT;
        } else {
            if (customClosePosition.indexOf("left") !== -1) {
                hLayoutRule = vrtcal.androidLayoutRules.ALIGN_PARENT_LEFT;
            } else if (customClosePosition.indexOf("right") !== -1) {
                hLayoutRule = vrtcal.androidLayoutRules.ALIGN_PARENT_RIGHT;
            }

            if (customClosePosition.indexOf("top") !== -1) {
                vLayoutRule = vrtcal.androidLayoutRules.ALIGN_PARENT_TOP;
            } else if (customClosePosition.indexOf("bottom") !== -1) {
                vLayoutRule = vrtcal.androidLayoutRules.ALIGN_PARENT_BOTTOM;
            }
        }

        return {x: x, y: y,
                width: resizeProperties.width, height: resizeProperties.height,
                offsetX: resizeProperties.offsetX, offsetY: resizeProperties.offsetY,
                allowOffscreen: allowOffscreen, hLayoutRule: hLayoutRule, vLayoutRule: vLayoutRule};
    };

    vrtcal.sizeEquals = function(size, other) {
        return (size.width === other.width && size.height === other.height);
    };

    vrtcal.sdk = {
        init: function(adType, layoutRules) {
            vrtcal.adType = adType;
            vrtcal.androidLayoutRules = layoutRules;

            vrtcal.expandProperties = {isModal: true};
            vrtcal.orientationProperties = {allowOrientationChange: true, forceOrientation: "none"};
            vrtcal.resizeProperties = {customClosePosition: "top-right", allowOffscreen: true};
        },

        onAdCollapsed: function() {
            var currentPosition = vrtcal.getCurrentPosition();
            vrtcal.state = "default";
            if (vrtcal.sizeEquals(vrtcal.previousSize, currentPosition) === false) {
                vrtcal.fireEvent("sizeChange", currentPosition.width, currentPosition.height);
                vrtcal.previousSize = {width: currentPosition.width, height: currentPosition.height};
            }
            vrtcal.fireEvent("stateChange", vrtcal.state);
        },

        onAdDismissed: function() {
            vrtcal.state = "hidden";
            if (vrtcal.sizeEquals(vrtcal.previousSize, {width: 0, height: 0}) === false) {
                vrtcal.fireEvent("sizeChange", 0, 0);
                vrtcal.previousSize = {width: 0, height: 0};
            }
            vrtcal.fireEvent("stateChange", vrtcal.state);
        },

        onAdExpanded: function() {
            var currentPosition = vrtcal.getCurrentPosition();
            vrtcal.state = "expanded";
            if (vrtcal.sizeEquals(vrtcal.previousSize, currentPosition) === false) {
                vrtcal.fireEvent("sizeChange", currentPosition.width, currentPosition.height);
                vrtcal.previousSize = {width: currentPosition.width, height: currentPosition.height};
            }
            vrtcal.fireEvent("stateChange", vrtcal.state);
        },

        onAdFailedToExpand: function(errorMessage) {
            vrtcal.fireErrorEvent(errorMessage, "expand");
        },

        onAdResized: function() {
            var currentPosition = vrtcal.getCurrentPosition();
            vrtcal.state = "resized";
            if (vrtcal.sizeEquals(vrtcal.previousSize, currentPosition) === false) {
                vrtcal.fireEvent("sizeChange", currentPosition.width, currentPosition.height);
                vrtcal.previousSize = {width: currentPosition.width, height: currentPosition.height};
            }
            vrtcal.fireEvent("stateChange", vrtcal.state);
        },

        onAdSizeChangedDueToDeviceRotation: function() {
            var currentPosition = vrtcal.getCurrentPosition();
            if (vrtcal.sizeEquals(vrtcal.previousSize, currentPosition) === false) {
                vrtcal.fireEvent("sizeChange", currentPosition.width, currentPosition.height);
                vrtcal.previousSize = {width: currentPosition.width, height: currentPosition.height};
            }
        },

        onExposureUpdated: function(visibleX, visibleY, visibleWidth, visibleHeight) {
            var lastVisibleArea = 0;
            if (!vrtcal.isEmpty(vrtcal.currentExposure)
                    && !vrtcal.isEmpty(vrtcal.currentExposure.width)
                    && !vrtcal.isEmpty(vrtcal.currentExposure.height)) {
                lastVisibleArea = vrtcal.currentExposure.width * vrtcal.currentExposure.height;
            }

            vrtcal.currentExposure = {x: visibleX, y: visibleY, width: visibleWidth, height: visibleHeight};
            var visibleArea = visibleWidth * visibleHeight;

            var currentPosition = mraid.getCurrentPosition();
            var containerArea = currentPosition.width * currentPosition.height;
            var exposedPercentage = (containerArea > 0) ? (100 * visibleArea / containerArea) : 0;
            var visibleRectangle = {x: visibleX, y: visibleY, width: visibleWidth, height: visibleHeight};
            if (exposedPercentage > 0) {
                vrtcal.isAdShown = true;
            }
            vrtcal.fireEvent("exposureChange", exposedPercentage, visibleRectangle, null);

            if ((lastVisibleArea == 0 && visibleArea > 0)
                || (lastVisibleArea > 0 && visibleArea == 0)) {
                vrtcal.fireEvent("viewableChange", (visibleArea > 0));
            }
        },

        onHostReady: function(defaultPosition) {
            vrtcal.defaultPosition = defaultPosition;
            vrtcal.previousSize = defaultPosition;

            if (vrtcal.isTwopartExpandable === false) {
                vrtcal.state = "default";
                vrtcal.fireEvent("stateChange", vrtcal.state);
            }
            vrtcal.fireEvent("ready");
        },

        onVolumeChanged: function(volumePercent) {
            vrtcal.fireEvent("audioVolumeChange", volumePercent);
        }
    };
})();


(function() {
    mraid.close = function() {
        if (vrtcal.state === "loading") {
            vrtcal.log("Cannot close when ad is still loading");
        } else if (vrtcal.state === "default" && vrtcal.adType === "BANNER") {
            vrtcal.log("Closing of banner ad is currently not supported");
        } else if (vrtcal.state === "default" && vrtcal.adType === "INTERSTITIAL") {
            VrtcalMraidJsBridge.dismissAd();
        } else if (vrtcal.state === "expanded") {
            VrtcalMraidJsBridge.collapseExpandedAd();
        } else if (vrtcal.state === "resized") {
            VrtcalMraidJsBridge.collapseResizedAd();
        } else if (vrtcal.state === "hidden") {
            vrtcal.log("Cannot close an ad that is already hidden");
        }
    };

    mraid.createCalendarEvent = function(calendarEvent) {
        if (vrtcal.hasRecentUserInteraction() === false) {
            vrtcal.fireErrorEvent("Cannot create calendar event without user interaction", "createCalendarEvent");
            return;
        }

        var success = mraid.supports("calendar");
        if (success === false) {
            vrtcal.fireErrorEvent("Permission to create calendar event is not granted", "createCalendarEvent");
            return;
        }

        var intentAction = "android.intent.action.INSERT";
        var intentDataUri = "content://com.android.calendar/events";
        var intentExtras = {
                calendar_id: {value: 1, type: "Long"},
                title: {value: calendarEvent.summary, type: "String"},
                description: {value: calendarEvent.description, type: "String"},
                eventLocation: {value: calendarEvent.location, type: "String"},
                beginTime: {value: (new Date(calendarEvent.start)).getTime(), type: "Long"},
                endTime: {value: (new Date(calendarEvent.end)).getTime(), type: "Long"},
                transparency: {value: calendarEvent.transparency, type: "String"},
                eventTimezone: {value: (new Date()).getTimezoneOffset(), type: "Integer"}
        };

        // Determine recurrence
        if (!vrtcal.isEmpty(calendarEvent.recurrence)) {
            var recurrence = vrtcal.formatCalendarEventRecurrence(calendarEvent.recurrence);
            if (recurrence !== "") {
                intentExtras.rrule = {value: recurrence, type: String};
            }
        }

        var idStr = VrtcalMraidJsBridge.startActivity(intentAction, intentDataUri, JSON.stringify(intentExtras));

        if (!vrtcal.isEmpty(idStr)) {
            var id = Number(idStr);

            if (!vrtcal.isEmpty(calendarEvent.reminder)) {
                // Calculate reminder time (in minutes before start time)
                var reminder = (new Date(calendarEvent.start)).getTime() - (new Date(calendarEvent.reminder)).getTime();
                reminder = reminder / 1000 / 60;

                var uri = "content://com.android.calendar/reminders";
                var contentValues = {
                        minutes: {value: reminder, type: "Long"},
                        method: {value: 1, type: "Integer"}
                };
                VrtcalMraidJsBridge.updateContentResolver(uri, "event_id=" + id, JSON.stringify(contentValues));
            }
        } else {
            vrtcal.fireErrorEvent("Unable to create calendar event due to errors", "createCalendarEvent");
        }
    };

    mraid.getCurrentAppOrientation = function() {
        var screenSize = vrtcal.getScreenSize();
        var orientation = (screenSize.width > screenSize.height)
                ? "landscape" : "portrait";
        var locked = (VrtcalMraidJsBridge.isAutoRotateEnabled() === false);
        return {orientation: orientation, locked: locked};
    };

    mraid.getCurrentPosition = function() {
        return vrtcal.getCurrentPosition();
    };

    mraid.getDefaultPosition = function() {
        return vrtcal.defaultPosition;
    };

    mraid.expand = function(url) {
        vrtcal.log("Attempting to expand with expandProperties " + JSON.stringify(mraid.getExpandProperties()));

        if (vrtcal.hasRecentUserInteraction() === false) {
            vrtcal.fireErrorEvent("Cannot expand ad without user interaction", "expand");
            return;
        }

        if (vrtcal.state === "loading") {
            vrtcal.log("Cannot expand when ad is still loading");
        } else if (vrtcal.state === "default" && vrtcal.adType === "BANNER") {
            vrtcal.expandAd(url);
        } else if (vrtcal.state === "default" && vrtcal.adType === "INTERSTITIAL") {
            vrtcal.log("Interstitial ads cannot be expanded");
        } else if (vrtcal.state === "expanded") {
            vrtcal.log("Cannot expand an already expanded ad", "expand");
        } else if (vrtcal.state === "resized") {
            vrtcal.expandAd(url);
        } else if (vrtcal.state === "hidden") {
            vrtcal.log("Cannot expand an ad that is already hidden");
        }
    };

    mraid.getExpandProperties = function() {
        var result = {};

        result.useCustomClose = vrtcal.useCustomClose;
        result.isModal = true;

        var screenSize = vrtcal.getScreenSize();
        if (!vrtcal.isEmpty(vrtcal.expandProperties) && !vrtcal.isEmpty(vrtcal.expandProperties.width)) {
            result.width = vrtcal.expandProperties.width;
        } else {
            result.width = screenSize.width;
        }
        if (!vrtcal.isEmpty(vrtcal.expandProperties) && !vrtcal.isEmpty(vrtcal.expandProperties.height)) {
            result.height = vrtcal.expandProperties.height;
        } else {
            result.height = screenSize.height;
        }

        return result;
    };

    mraid.setExpandProperties = function(value) {
        if (vrtcal.isEmpty(value)) {
            return;
        }

        if (!vrtcal.isEmpty(value.width)) {
            vrtcal.expandProperties.width = value.width;
        }
        if (!vrtcal.isEmpty(value.height)) {
            vrtcal.expandProperties.height = value.height;
        }
        if (!vrtcal.isEmpty(value.useCustomClose)) {
            vrtcal.useCustomClose = value.useCustomClose;
        }
    };

    mraid.getLocation = function() {
        if (mraid.supports("location") === false) {
            vrtcal.log("Cannot get location because it is not supported");
            vrtcal.fireErrorEvent("-1", "getLocation");
            return null;
        }

        var locationStr = VrtcalMraidJsBridge.getLocation();
        if (!vrtcal.isEmpty(locationStr)) {
            var location = JSON.parse(locationStr);
            if (!vrtcal.isEmpty(location)) {
                var lastfix = Math.round(((new Date()).getTime() - location.time) / 1000);
                return {lat: location.lat, lon: location.lon, type: 1, accuracy: location.accuracy, lastfix: lastfix};
            }
        }

        vrtcal.fireErrorEvent("-1", "getLocation");
        return null;
    };

    mraid.getMaxSize = function() {
        return vrtcal.getMaxSize();
    };

    mraid.open = function(url) {
        if (vrtcal.hasRecentUserInteraction() === false) {
            vrtcal.fireErrorEvent("Cannot open URL without user interaction", "open");
            return;
        }

        VrtcalMraidJsBridge.openUrl(url);
    };

    mraid.getOrientationProperties = function() {
        return vrtcal.orientationProperties;
    };

    mraid.setOrientationProperties = function(value) {
        if (vrtcal.isEmpty(value)) {
            return;
        }
        if (!vrtcal.isEmpty(value.allowOrientationChange)) {
            vrtcal.orientationProperties.allowOrientationChange = value.allowOrientationChange;
        }
        if (!vrtcal.isEmpty(value.forceOrientation)) {
            vrtcal.orientationProperties.forceOrientation = value.forceOrientation;
        }

        if (vrtcal.adType === "INTERSTITIAL" || vrtcal.state === "expanded") {
            VrtcalMraidJsBridge.setOrientationRules(vrtcal.orientationProperties.allowOrientationChange,
                    vrtcal.orientationProperties.forceOrientation);
        }
    };

    mraid.getPlacementType = function() {
        if (vrtcal.adType === "INTERSTITIAL") {
            return "interstitial";
        } else {
            return "inline";
        }
    };

    mraid.playVideo = function() {
        vrtcal.fireErrorEvent("This SDK supports HTML5 video player instead of native video player", "playVideo");
    };

    mraid.resize = function() {
        if (vrtcal.hasRecentUserInteraction() === false) {
            vrtcal.fireErrorEvent("Cannot resize ad without user interaction", "resize");
            return;
        }

        if (vrtcal.state === "loading") {
            vrtcal.log("Cannot resize when ad is still loading");
        } else if (vrtcal.state === "default" && vrtcal.adType === "BANNER") {
            vrtcal.resizeAd();
        } else if (vrtcal.state === "default" && vrtcal.adType === "INTERSTITIAL") {
            vrtcal.log("Interstitial ads cannot be resized");
        } else if (vrtcal.state === "expanded") {
            vrtcal.fireErrorEvent("Expanded ads cannot be resized", "resize");
        } else if (vrtcal.state === "resized") {
            vrtcal.resizeAd();
        } else if (vrtcal.state === "hidden") {
            vrtcal.log("Cannot resize an ad that is already hidden");
        }
    };

    mraid.getResizeProperties = function() {
        return vrtcal.resizeProperties;
    };

    mraid.setResizeProperties = function(value) {
        var validationResult = vrtcal.validateResizeProperties(value);
        if (!vrtcal.isEmpty(validationResult.error)) {
            vrtcal.fireErrorEvent("Cannot set resizeProperties: " + validationResult.error, "setResizeProperties");
            return;
        }

        vrtcal.resizeProperties = value;
    };

    mraid.getScreenSize = function() {
        return vrtcal.getScreenSize();
    };

    mraid.storePicture = function(uri) {
        if (vrtcal.hasRecentUserInteraction() === false) {
            vrtcal.fireErrorEvent("Cannot store picture without user interaction", "storePicture");
            return;
        }

        if (mraid.supports("storePicture") !== true) {
            vrtcal.fireErrorEvent("storePicture() is not supported", "storePicture");
            return;
        }

        // Prompt user for permission.  Need to set ad visibility to 0 while dialog is showing.
        var exposure = vrtcal.currentExposure;
        vrtcal.sdk.onExposureUpdated(0, 0, 0, 0);
        var permissionGranted =
                VrtcalMraidJsBridge.requestUserPermission("Store Picture", "Allow ad to store picture in photo album?");
        vrtcal.sdk.onExposureUpdated(exposure.x, exposure.y, exposure.width, exposure.height);
        if (permissionGranted !== true) {
            vrtcal.fireErrorEvent("User denied permission to store picture", "storePicture");
            return;
        }

        var success = VrtcalMraidJsBridge.storePicture(uri);
        if (success !== true) {
            vrtcal.fireErrorEvent("Error occurred trying to save picture to gallery", "storePicture");
            return;
        }
    };

    mraid.supports = function(feature) {
        if (feature === "vpaid") {
            return false;
        } else {
            return VrtcalMraidJsBridge.supports(feature);
        }
    };

    mraid.unload = function() {
        if (vrtcal.adType === "BANNER") {
            if (mraid.getState() === "expanded") {
                VrtcalMraidJsBridge.dismissExpandedAd();
            } else {
                VrtcalMraidJsBridge.dismissAd();
            }
        } else {
            if (vrtcal.isAdShown === true) {
                VrtcalMraidJsBridge.dismissAd();
            } else {
                VrtcalMraidJsBridge.failAd();
            }
        }
    };

    mraid.useCustomClose = function(value) {
        vrtcal.useCustomClose = value;
    };

    mraid.getVersion = function() {
        return window.MRAID_ENV.version;
    };

    mraid.isViewable = function() {
        var visibleArea = 0;
        if (!vrtcal.isEmpty(vrtcal.currentExposure)) {
            visibleArea = vrtcal.currentExposure.width * vrtcal.currentExposure.height;
        }
        return (visibleArea > 0);
    };
})();
