'use strict';

var device;
var boolGetStateOfWemo = false;
var commandVal;
var boolIntialState = false;
var commandTimeout;

var sendDeviceCommand = function () {
  try {
    sendRemoteButtonPressed(151, "Belkin Switch", command, "inapp");
  } catch (error) {
    log("ERROR", "sendRemoteButtonPressed is not defined");
  }
  getWemoState();
}

var getWemoState = function () {
  boolGetStateOfWemo = true;
  var url = `http://${device.ip}:${device.port}/upnp/control/basicevent1`;
  var method = 'POST';
  var soapdata = "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n<s:Envelope xmlns:s=\"http://schemas" +
      ".xmlsoap.org/soap/envelope/\" s:encodingStyle=\"http://schemas.xmlsoap.org/soap/" +
      "encoding/\">\n <s:Body>\n  <u:GetBinaryState xmlns:u==\"urn:Belkin:service:basic" +
      "event:1\">\n  </u:GetBinaryStat>\n </s:Body>\n</s:Envelope>\n";
  log("DEBUG", "soapdata: " + soapdata);
  var headersJsonMap = JSON.stringify({'SOAPACTION': '"urn:Belkin:service:basicevent:1#GetBinaryState"', 'Content-Type': 'text/xml; charset=UTF-8', 'Content-Length': soapdata.length});
  removeCommandTimeout();
  commandTimeout = setTimeout(function () {
    log("ERROR", 'commandTimeout');
    toggleAlertMessage('block');
  }, 2000);
  try {
    Android.sendHttpRequest(method, headersJsonMap, soapdata, url);
  } catch (error) {
    log("ERROR", "Android interface not found");
  }
}

var sendRawCommand = function (state) {
  boolGetStateOfWemo = false;
  var test = "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n";
  var soapdata = "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n<s:Envelope xmlns:s=\"http://schemas" +
      ".xmlsoap.org/soap/envelope/\" s:encodingStyle=\"http://schemas.xmlsoap.org/soap/" +
      "encoding/\">\n <s:Body>\n  <u:SetBinaryState xmlns:u==\"urn:Belkin:service:basic" +
      "event:1\">\n   <BinaryState><#text>" + state + "</#text></BinaryState>\n  </u:SetBinaryState>\n </s:Body>\n</s:Envelope>\n";
  console.log(soapdata);
  var url = `http://${device.ip}:${device.port}/upnp/control/basicevent1`;
  var method = 'POST';
  var headersJsonMap = JSON.stringify({'SOAPACTION': '"urn:Belkin:service:basicevent:1#SetBinaryState"', 'Content-Type': 'text/xml; charset=UTF-8', 'Content-Length': soapdata.length});
  removeCommandTimeout();
  commandTimeout = setTimeout(function () {
    log("ERROR", 'commandTimeout');
    toggleAlertMessage('block');
  }, 2000);
  try {
    Android.sendHttpRequest(method, headersJsonMap, soapdata, url);
  } catch (error) {
    log("ERROR", "Android interface not found");
  }
}

var setWemoBinaryState = function (state) {
  log("DEBUG", "state: " + state);
  var data = (enable)
    ? 0
    : 1;
  commandVal = data;
  sendRawCommand(data);
}

var appendStyleSheet = function () {
  var style = document.createElement("style");
  document
    .head
    .appendChild(style);
  var sheet = style.sheet;
  sheet.insertRule('input:checked+.slider:before { -webkit-transform: translateX(66px);-ms-transform' +
      ': translateX(66px);transform: translateX(66px);outline: none; }',
  0);
}

var getHttpJsonResponse = function (response) {
  log("DEBUG", "getHttpJsonResponse : " + response);
  var headers,
    status,
    result,
    url,
    resultIndex,
    statusindex,
    resultVal;
  var extractedRespone = response.replace('="', "='");
  extractedRespone = extractedRespone.replace('""', "'\"");
  extractedRespone = extractedRespone.replace('"?', "'?");
  extractedRespone = extractedRespone.replace('"{', "{");
  extractedRespone = extractedRespone.replace('}"', "}");
  extractedRespone = response.replace(/\n/g, '');
  log("DEBUG", "extractedRespone: " + extractedRespone);
  resultIndex = extractedRespone.indexOf("result");
  statusindex = extractedRespone.indexOf(',"statusCode');
  resultVal = extractedRespone.substring(resultIndex + 'result":"'.length, statusindex);
  log("DEBUG", "resultVal: " + resultVal);
  var statusVal = extractedRespone
    .substring(statusindex)
    .split(":")[1];
  log("DEBUG", "statusVal:" + statusVal)
  var status = parseInt(statusVal.substring(0, statusVal.length - 1));
  log("DEBUG", "status:" + status);
  if (status === 200) {
    removeCommandTimeout();
    toggleAlertMessage("none")
    log("DEBUG", "Command Sent Successfull");
    if (boolGetStateOfWemo) {
      var regex = /<BinaryState>([0-1])<\/BinaryState>/gm;
      var match = regex.exec(resultVal);
      log("DEBUG", "match: " + match);

      if (match !== null) {
        var state = (match[1] == '1');
        if (boolIntialState) {
          boolIntialState = false;
          appendStyleSheet();
          document.getElementsByClassName("slider")[0].style.background = state
            ? '#2196F3'
            : '#ccc';
          return;
        }
        setWemoBinaryState(state);
      }
    } else {
      appendStyleSheet();
      document.getElementsByClassName("slider")[0].style.background = (commandVal == 0)
        ? '#ccc'
        : '#2196F3';
      sendInsightEvent(true, commandVal == 0
        ? 'off'
        : 'on');
    }
  } else {
    toggleAlertMessage("block");
    sendInsightEvent(false, commandVal == 0
      ? 'off'
      : 'on');
  }
}

var getHttpResponse = function (response) {}

var removeCommandTimeout = function () {
  if (commandTimeout) {
    clearTimeout(commandTimeout);
    commandTimeout = null;
  }
}

function initDeviceInfo() {
  console.log("Init Device Info");
  try {
    device = JSON.parse(getInfo('currentDevice'));
    log("DEBUG", JSON.stringify(device));
    if (device.modelNumber) {
      document
        .getElementById("deviceInfo")
        .innerHTML = `${device.brandName} - ${device.modelNumber}`;
    }
    boolIntialState = true;
    getWemoState();
  } catch (err) {
    log("ERROR", "Unable to initialize device info: " + err);
  }
}

function getInfo(value) {
  var info = "empty";
  switch (value) {
    case "currentDevice":
      info = Android.getCurrentDeviceInfo();
      break;
    case "currentActivity":
      info = Android.getCurrentActivityInfo();
      break;
    case "currentRoom":
      info = Android.getCurrentRoomInfo();
      break;
  }
  return info;
}