var device = {
    "ip": "192.168.1.1",
    "port": "8080",	 
	"contextId":151, 
    "source":"inapp"
};
var pressTimer;
var commandTimeout;

var commandMap = {
    'Power': "power",
    'Navigate_Up': "up",
    'Navigate_Down': "down",
    'Navigate_Left': "left",
    'Navigate_Right': "right",
    'Back': "back",
    'Select': "select",
    '1': "1",
    '2': "2",
    '3': "3",
    '4': "4",
    '5': "5",
    '6': "6",
    '7': "7",
    '8': "8",
    '9': "9",
    '0': "0",
    '.': "dash",
    'Channel_Up': "chanup",
    'Channel_Down': "chandown",
    'Enter': "enter",
    'Blue': "blue",
    'Red': "red",
    'Green': "green",
    'Yellow': "yellow",
    'Guide': "guide",
    'Info': "info",
    'Exit': "exit",
    'Menu': "menu",
    'Previous': "prev",
    'DVR': "list",
    'Active': "active",
    'Fast_Forward': "ffwd",
    'Rewind': "rew",
    'Play': "play",
    'Pause': "pause",
    'Stop': "stop",
    'Record': "record"
};

var startTimer = function(event, command) {
    pressTimer = setTimeout(function() {
        removeCommandTimeout();
        sendDirecTVCommand(event, command);
        startTimer(event, command);
    }, 100);
}

var clearTimer = function() {
    clearTimeout(pressTimer);
}

var sendDirecTVCommand = function(event, command){
    highlightingButtons(event, command);
    sendDeviceCommand(command, "inApp");
}

function sendDeviceCommand(command, location) {
    var deviceCommand = commandMap[command];
    if (deviceCommand === undefined) {
        log("ERROR", "Error! DirecTV DVR command undefined:" + deviceCommand);
    } else {
        if (device === undefined) {
            log("ERROR", "sendCommand error. device is undefined.");
        } else {
            sendIpCommand(deviceCommand);
            log("DEBUG", "DirecTV DVR send command:" + command);
        }
    }
}

var removeCommandTimeout = function() {
    if (commandTimeout) {
        clearTimeout(commandTimeout);
        commandTimeout = null;
    }
}

function sendIpCommand(command) {
    removeCommandTimeout();
    commandTimeout = setTimeout(function() {
        log("ERROR", 'commandTimeout');
        toggleAlertMessage('block');
    }, 1000);
    fetch("http://" + device.ip + ':8080/remote/processKey?key=' + command)
        .then(function(response) {
            return response.json();
        }).then(function(payload) {
            removeCommandTimeout();
            try {
                toggleAlertMessage('none');
                log("DEBUG", "DirecTV DVR sent status:" + payload.status.code + ", msg:" + payload.status.msg + ", commandResult:" + payload.status.commandResult + ", query:" + payload.status.query);
                if (payload.status.code == 200) {
                    log("DEBUG", "DirecTV DVR command sent:" + command);
                    //sendInsightEvent(true, command);
                    try{
                        sendCommandDetailSentEvent(true, command, device.contextId, getSource(device.contextId));
                    }catch(errors){
                        sendInsightEvent(true, command);
                    }
                } else {
                    log("DEBUG", "DirecTV DVR command sent failed." + command + " code:" + payload.status.code + " msg:" + payload.status.msg);
                    //sendInsightEvent(false, command);
                    try{
                        sendCommandDetailSentEvent(false, command, device.contextId, getSource(device.contextId));
                    }catch(errors){
                        sendInsightEvent(false, command);
                    }
                }
            } catch (err) {
                toggleAlertMessage('block');
                log("ERROR", "DirecTV DVR command sent failed:" + err);
                //sendInsightEvent(false, command);
                try{
                    sendCommandDetailSentEvent(false, command, device.contextId, getSource(device.contextId));
                }catch(errors){
                    sendInsightEvent(false, command);
                }
            }
        }).catch(function() {
            log("ERROR", "DirecTV DVR command sent failed.");
            toggleAlertMessage('block');
            //sendInsightEvent(false, command);
            try{
                sendCommandDetailSentEvent(false, command, device.contextId, getSource(device.contextId));
            }catch(errors){
                sendInsightEvent(false, command);
            }
        });
}


function initDeviceInfo() {
    try {
        device = JSON.parse(getInfo('currentDevice'));
        log("DEBUG", device.ip + " " + device.port);
        if (device.modelNumber) {
            if (device.friendlyName){
                document.getElementById("deviceInfo").innerHTML = `${device.modelNumber} - ${device.friendlyName}`;
            }
            else {
                document.getElementById("deviceInfo").innerHTML = `${device.brandName} - ${device.modelNumber}`;
            }
        }
    } catch (err) {
        log("ERROR", "Unable to initialize device info: " + err);
    }
}

function getInfo(value) {
    var info = "empty";
    switch (value) {
        case "currentDevice":
            info = Android.getCurrentDeviceInfo();
            break;
        case "currentActivity":
            info = Android.getCurrentActivityInfo();
            break;
        case "currentRoom":
            info = Android.getCurrentRoomInfo();
            break;
    }
    return info;
}

