'use strict';

var deviceIp;

var device = {
    "ip": "192.168.1.38",
    "port": "8080",	 
    "contextId":151, 
    "source":"inapp"
};
	 
var pressTimer;
var isCheckPair = true;
var isPaired = false;
var commandSent = '';
var commandSentLocation;
var connectionState = "";
var requestTimeout;
var isPairingTriggeredFromLockPanel = false;

var commandMap = {
    Menu: 22,
    Channel_Up: 27,
    Channel_Down: 28,
    PEL_CHSET: -1,
    Blue: 29,
    Red: 31,
    Green: 30,
    Yellow: 32,
    DVR: 'TVControl.DVR',
    Info: 45,
    Anynet: 'TVControl.Extra.AnyNet',
    Tools: 'TVControl.Extra.Tools',
    Back: 23,
    Skip_Forward: 38,
    Skip_Back: 39,
    Navigate_Up: 12,
    Navigate_Down: 13,
    Navigate_Left: 14,
    Navigate_Right: 15,
    Select: 20,
    Home: 21,
    1: 3,
    2: 4,
    3: 5,
    4: 6,
    5: 7,
    6: 8,
    7: 9,
    8: 10,
    9: 11,
    0: 2,
    11: 'KeyControl.Num11',
    12: 'KeyControl.Num12',
    Dot_DASh: 402,
    Volume_Up: 24,
    Volume_Down: 25,
    Mute: 26,
    Epg: 44,
    LiveTV: 43,
    Input: 47,
    Antenna: 'InputControl.Antenna',
    HDMI: 'InputControl.HDMI',
    HDMI1: 'InputControl.HDMI1',
    HDMI2: 'InputControl.HDMI2',
    HDMI3: 'InputControl.HDMI3',
    HDMI4: 'InputControl.HDMI4',
    TV: 43,
    DVI: 'InputControl.DVI',
    AV1: 410,
    Svideo1: 416,
    Component1: 'InputControl.COMPONENT1',
    Component2: 'InputControl.COMPONENT2',
    PowerOff: 'PowerControl.On',
    PowerOn: 'PowerControl.Off',
    Exit: 412,
    Power: 1,
    Rewind: 37,
    Stop: 35,
    Play: 33,
    Fast_Forward: 36,
    Record: 40,
    Pause: 34
};

var removeRequestTimeout = function() {
    if (requestTimeout) {
        clearTimeout(requestTimeout);
        requestTimeout = null;
    }
}

function initDeviceInfo() {
    try {
        device = JSON.parse(getInfo('currentDevice'));
        log("DEBUG", "Device: " + JSON.stringify(device));
        log("DEBUG", device.ip + " " + device.port);
        log("DEBUG", "On initDeviceInfo" + device.contextId);
        log("DEBUG", "On initDeviceInfo" + device.source);
        if (device.modelNumber) {
            if (device.friendlyName){
                document.getElementById("deviceInfo").innerHTML = `${device.modelNumber} - ${device.friendlyName}`;
            }
            else {
                document.getElementById("deviceInfo").innerHTML = `${device.brandName} - ${device.modelNumber}`;
            }
        }
        var localStoredData = localStorage.getItem(device.ip);
        log("DEBUG", "localStoredData: " + localStoredData);
        if (localStoredData) {
            try {
                localStoredData = JSON.parse(localStoredData);
                log("DEBUG", "There is localStoredData");
                log("DEBUG", "commandSentLocation : " + localStoredData.commandSentLocation);
                if (localStoredData.commandSentLocation && localStoredData.commandSentLocation === 'widget') {
                    let commandSent = localStoredData.commandSent;
                    log("DEBUG", "commandSentLocation : " + commandSent);
                    localStorage.removeItem(device.ip);
                    isPairingTriggeredFromLockPanel = true;
                    sendDeviceCommand(commandSent, getSource(device.contextId));
                }
            } catch (e) {
                log("DEBUG", "unable to parse JSON");
            }

        }
    } catch (err) {
        log("ERROR", "Unable to initialize device info: " + err);
    }
}

function helloExchange(pairingValue) {
    try {
        log("DEBUG", "Sending Pairing request code client: " + pairingValue);            
        var method = "POST";
        var userAgent = "UDAP/2.0";
        var payload = '<?xml version="1.0" encoding="utf-8"?><envelope><api type="pairing"><name>hello</name><value>' + pairingValue + '</value></api></envelope>';
        var url = `http://${device.ip}:8080/udap/api/pairing`;
        var contentType = "text/xml";
        connectionState = "pair";
        var headersJsonMap = JSON.stringify({ "User-Agent": "UDAP/2.0", "Content-Type": "text/xml" });
        removeRequestTimeout();
        requestTimeout = setTimeout(function() {
            log("ERROR", 'requestTimeout');
            // sendPairingResultEvent(false, device.type, device.brandName);
            toggleAlertMessage('block');
        }, 2000);
        var values;
        try {
            values = getContextIdAndSource(device.contextId);
            log("DEBUG", "Reading ContextId from client: " + values.contextId);
        } catch (error) {
            values = {
                "contextId": 151,
                "source": "inapp"
            }
            log("DEBUG", "Error Reading ContextId from client: " + values.contextId);            }
        

        Android.sendHttpRequest(method, headersJsonMap, payload, url);
    } catch (err) {
        log("ERROR", "sendHttpRequest: No Android Interface:" + err);
    }
}

var storeAndSendPairingCode = function() {
    var pairingCode = document.getElementById("pairingValue").value;
    log("DEBUG", "store And Send Pairing Code : " + pairingCode);
    if (pairingCode !== '' || typeof pairingCode !== 'undefined') {
        log("DEBUG", "valid pairingCode: " + pairingCode);
        localStorage.setItem(device.ip, pairingCode);	
        log("DEBUG", "pairingCode stored in local storage: " + pairingCode);
        document.getElementById("pairingValue").value = '';
        sendPairingEvent(device.type, device.brandName, "STARTED", "CONTROL_PAD");
        //log("DEBUG", "Sending Pariring Event from storeAndSendPairingCode function");
        helloExchange(pairingCode);
    }
}

var checkIsPairingDone = function(deviceIp) {
    removeRequestTimeout();
    log("DEBUG", "deviceIp:" + device.ip);
    var storedPairingCode = localStorage.getItem(device.ip);
    log("DEBUG", "storedPairingCode:" + storedPairingCode);
    if (storedPairingCode !== null) {
        log("DEBUG", "pairingCode: not null");
        if (document.getElementById('pairingCodeBox')) {
            document.getElementById('pairingCodeBox').style.display = 'none';
            document.getElementById('rectangle').style.display = 'block';
        }
        helloExchange(storedPairingCode);
    } else {
        if (commandSentLocation === 'widget') {
            try {
                Android.openDeviceRemote(device.id);
            } catch (e) {
                log("DEBUG", "Android interface not found")
            }
        } else {
            log("DEBUG", "pairingCode: null");
            setTimeout(function(){
                document.getElementById("pairingInputContainer").style.display = 'block';
                document.getElementById("alert-message").style.display = 'none';
                document.getElementById('pairingCodeBox').style.display = 'block';
                document.getElementById('rectangle').style.display = 'none';
                if(!isPairingTriggeredFromLockPanel){
                    sendPairingEvent(device.type, device.brandName, "TRIGGERED", "CONTROL_PAD");   
                    //log("DEBUG", "Sending Pariring Event from checkIsPairingDone function");
                }
                showPairingKeyRequest();
            }, 100);
        }
    }
}

var closeDialog = function() {
    removeRequestTimeout();
    document.getElementById("pairingInputContainer").style.display = 'none';
    document.getElementById("alert-message").style.display = 'none';
    document.getElementById("pairingCodeBox").style.display = 'none';
    document.getElementById("rectangle").style.display = 'block';
}

function showPairingKeyRequest() {
    try {
        var method = "POST";
        var userAgent = "UDAP/2.0";
        var payload = '<?xml version="1.0" encoding="utf-8"?><envelope><api type="pairing"><name>showKey</name></api></envelope>';
        var url = `http://${device.ip}:8080/udap/api/pairing`;
        var contentType = "text/xml";
        var headersJsonMap = JSON.stringify({ "User-Agent": "UDAP/2.0", "Content-Type": "text/xml" });
        connectionState = "showKey";
        Android.sendHttpRequest(method, headersJsonMap, payload, url);
    } catch (err) {
        log("ERROR", "sendHttpRequest: No Android Interface:" + err);
    }
}


var startTimer = function(event, command) {
    pressTimer = setTimeout(function() {
        sendLgCommand(event, command);
        startTimer(event, command);
    }, 100);
}

var clearTimer = function() {
    clearTimeout(pressTimer);
}

var sendLgCommand = function(event, command) {
    log("DEBUG", "sendLgCommand: " + command);
    highlightingButtons(event, command);

    
    sendDeviceCommand(command, getSource(device.contextId));
}

var checkToOpenDeviceRemoteInApp = function(command, commandSentLocation) {
    try {
        var storedData = localStorage.getItem(device.ip);
        log("DEBUG", "storedData: " + storedData);
        if (!storedData) {
            storedData = {};
            log("DEBUG", "storedData is null");
            storedData.commandSentLocation = commandSentLocation;
            storedData.commandSent = command;
            localStorage.setItem(device.ip, JSON.stringify(storedData));
            commandSentLocation = '';
            Android.openDeviceRemote(device.id);
            log("DEBUG", "openDeviceRemoteSent");
            return true;
        }
        log("DEBUG", "openDeviceRemoteSent");
    } catch (e) {
        log("DEBUG", "Android interface not found")
    }
    return false;
}

var sendDeviceCommand = function(command, location) {

    var values;
    try {
        values = getContextIdAndSource(device.contextId);
    } catch (error) {
        values = {
            "contextId": 151,
            "source": "inapp"			
        }
        log("DEBUG", "Error Sending ContextId from JS code on sendDeviceCommand: " + values.contextId);
    }
    log("DEBUG", "isPaired: " + isPaired);
    commandSentLocation = values.source;
    commandSent = command;
    if (location === 'widget') {
        if (checkToOpenDeviceRemoteInApp(command, location)) {
            sendPairingEvent(device.type, device.brandName, "TRIGGERED", "WIDGET");
            //log("DEBUG", "Sending Pariring Event from sendDeviceCommand function");
            return;
        }
        log("DEBUG", "checking recursive");
    }
    removeRequestTimeout();
    requestTimeout = setTimeout(function() {
        log("ERROR", 'requestTimeout');
        toggleAlertMessage('block');
    }, 2000);
    if (!isPaired) {
        checkIsPairingDone();
        return;
    }
    log("DEBUG", "sendCommand: " + command);
    log("DEBUG", "isPaired: " + isPaired);
    try {
        var method = "POST";
        var headersJsonMap = JSON.stringify({ "User-Agent": "UDAP/2.0", "Content-Type": "text/xml" });
        var payload = '<?xml version="1.0" encoding="utf-8"?><envelope><api type="command"><name>HandleKeyInput</name><value>' + commandMap[command] + '</value></api></envelope>';
        var url = `http://${device.ip}:8080/udap/api/command`;
        Android.sendHttpRequest(method, headersJsonMap, payload, url);
    } catch (err) {
        log("ERROR", "sendHttpRequest: No Android Interface:" + err);
    }
}

var disconnect = function() {
    toggleAlertMessage("block");
}

function getHttpResponse(value) {
    log("DEBUG", " HttpResponse:" + value);
    var httpResponseValues = value.split('|');
    var statusCode = httpResponseValues[1];
    var response = httpResponseValues[2];
    var url = httpResponseValues[0];
    removeRequestTimeout();
    log("DEBUG", " statusCode:" + statusCode);
    log("DEBUG", " url:" + url);
    log("DEBUG", " deviceIp:" + device.ip);
    log("DEBUG", 'testurl: http://' + device.ip + ':8080/udap/api/pairing');
    log("DEBUG", " response:" + response);
    log("DEBUG", " isPaired:" + isPaired);
    log("DEBUG", "device url: " + url);
    switch (url) {
        case 'http://' + device.ip + ':8080/udap/api/pairing':
            log("DEBUG", " pairing statusCode: " + statusCode);
            switch (connectionState) {
                case "showKey":
                    connectionState = '';
                    if (parseInt(statusCode) === 200) {
                        sendPairingEvent(device.type, device.brandName, "SHOWN", "CONTROL_PAD");
                        //log("DEBUG", "Sending Pariring Event from getHttpResponse function when status code: showKey" + statusCode);
                    }else{
                        sendPairingResultEvent(false, device.type, device.brandName);
                        //log("DEBUG", "Sending Pariring Event from getHttpResponse function when status code is not : showKey" + statusCode);
                    }
                    return;
                case "pair":
                    connectionState = '';
                    if (parseInt(statusCode) === 200) {
                        isPaired = true;
                        //sendPairingResultEvent(true, device.type, device.brandName); //removed to avoid repeted pairing events
                        //log("DEBUG", "Sending Pariring Event from getHttpResponse function when status code is not : pair" + statusCode);
                        toggleAlertMessage("none");
                        if (document.getElementById("pairingInputContainer")) {
                            document.getElementById("pairingInputContainer").style.display = 'none';
                            document.getElementById("alert-message").style.display = 'none';
                            document.getElementById("pairingCodeBox").style.display = 'none';
                            document.getElementById("rectangle").style.display = 'block';
                        }
                        sendDeviceCommand(commandSent, getSource(device.contextId));
                    } else if (parseInt(statusCode) === 401) {
                        isPaired = false;
                        sendPairingResultEvent(false, device.type, device.brandName);
                        //log("DEBUG", "Sending Pariring Event from getHttpResponse function when status code is not : " + statusCode);
                        localStorage.removeItem(device.ip);
                        checkIsPairingDone(device.ip);
                    } else if (parseInt(statusCode) === 400){
                        isPaired = false;
                        sendPairingResultEvent(false, device.type, device.brandName);
                        //log("DEBUG", "Sending Pariring Event from getHttpResponse function when status code is not : " + statusCode);
                        toggleAlertMessage("block");
                    }
                    break;
            }
            break;
        case 'http://' + device.ip + ':8080/udap/api/command':
            log("DEBUG", " sendCommand statusCode: " + statusCode + typeof statusCode);
            log("DEBUG", " sendCommand: " + commandSent);
            if (parseInt(statusCode) === 200) {
                toggleAlertMessage("none");
                try {
                    log("DEBUG", "Calling  sendCommandDetailSentEvent reached here");
                    sendCommandDetailSentEvent(true, commandSent, device.contextId, getSource(device.contextId));
                    log("DEBUG", "Sending ContextId from JS code: " + device.contextId);
                } catch (error) {
                    sendInsightEvent(true, commandSent);                    
                    log("DEBUG", "Sending sendInsightEvent: true: " + commandSent + " statusCode:" + statusCode );
                }
            } else if (parseInt(statusCode) === 401) {
                try {
                    values = getContextIdAndSource(device.contextId);
                    sendCommandDetailSentEvent(false, commandSent, device.contextId, getSource(device.contextId));
                    log("DEBUG", "Sending ContextId from JS code: " + values.contextId);
                } catch (error) {
                    sendInsightEvent(false, commandSent);    
                    log("DEBUG", "Sending sendInsightEvent: true: " + commandSent + " statusCode:" + statusCode );
                }
                isPaired = false;
                checkIsPairingDone();
            } else {
                try {
                    values = getContextIdAndSource(device.contextId);
                    sendCommandDetailSentEvent(false, commandSent, device.contextId, getSource(device.contextId));
                    log("DEBUG", "Sending ContextId from JS code: " + values.contextId);
                } catch (error) {
                    sendInsightEvent(false, commandSent);    
                    log("DEBUG", "Sending sendInsightEvent: true: " + commandSent + " statusCode:" + statusCode );
                }
                toggleAlertMessage("block");
            }
            break;
    }
}

function getInfo(value) {
    var info = "empty";
    switch (value) {
        case "currentDevice":
            info = Android.getCurrentDeviceInfo();
            break;
        case "currentActivity":
            info = Android.getCurrentActivityInfo();
            break;
        case "currentRoom":
            info = Android.getCurrentRoomInfo();
            break;
    }
    return info;
}