'use strict';


var commandVal;	

var device = {
    "ip": "192.168.1.255",
    "port": "55000",	 
	"contextId":151, 
    "source":"inapp"
};
var pressTimer;
var commandSentLocation;
var pairingTimeout;
var commandTimeout;
var commandStatus = '';
// urls
var URL_NETWORK = '/nrc/control_0';
var URL_RENDERING = '/dmr/control_0';

// urns
var URN_NETWORK = 'panasonic-com:service:p00NetworkControl:1';
var URN_RENDERING = 'schemas-upnp-org:service:RenderingControl:1';

// actions
var ACTION_SENDKEY = 'X_SendKey';
var ACTION_GETVOLUME = 'GetVolume';
var ACTION_SETVOLUME = 'SetVolume';
var ACTION_GETMUTE = 'GetMute';
var ACTION_SETMUTE = 'SetMute';

// args
var ARG_SENDKEY = 'X_KeyEvent';
var commandToCapability = {
    'Power': 'NRC_POWER-ONOFF',
    'Volume_Up': 'NRC_VOLUP-ONOFF',
    'Volume_Down': 'NRC_VOLDOWN-ONOFF',
    'Mute': 'NRC_MUTE-ONOFF',
    'Input': 'NRC_CHG_INPUT-ONOFF',
    'Apps': 'NRC_APPS-ONOFF',
    'Pause': 'NRC_PAUSE-ONOFF',
    'Play': 'NRC_PLAY-ONOFF',
    'Skip_Back': 'NRC_SKIP_PREV-ONOFF',
    'Skip_Forward': 'NRC_SKIP_NEXT-ONOFF',
    'Rewind': 'NRC_REW-ONOFF',
    'Fast_Forward': 'NRC_FF-ONOFF',
    'Record': 'NRC_REC-ONOFF',
    'Menu': 'NRC_MENU-ONOFF',
    'Home': 'NRC_HOME-ONOFF',
    'Info': 'NRC_INFO-ONOFF',
    'Red': 'NRC_RED-ONOFF',
    'Green': 'NRC_GREEN-ONOFF',
    'Blue': 'NRC_BLUE-ONOFF',
    'Yellow': 'NRC_YELLOW-ONOFF',
    'Select': 'NRC_ENTER-ONOFF',
    'Channel_Up': 'NRC_CH_UP-ONOFF',
    'Channel_Down': 'NRC_CH_DOWN-ONOFF',
    'Navigate_Left': 'NRC_LEFT-ONOFF',
    'Navigate_Right': 'NRC_RIGHT-ONOFF',
    'Navigate_Up': 'NRC_UP-ONOFF',
    'Navigate_Down': 'NRC_DOWN-ONOFF',
    'Dot_DASh': 'NRC_PROG-ONOFF',
    'Stop': 'NRC_STOP-ONOFF',
    '0': 'NRC_D0-ONOFF',
    '1': 'NRC_D1-ONOFF',
    '2': 'NRC_D2-ONOFF',
    '3': 'NRC_D3-ONOFF',
    '4': 'NRC_D4-ONOFF',
    '5': 'NRC_D5-ONOFF',
    '6': 'NRC_D6-ONOFF',
    '7': 'NRC_D7-ONOFF',
    '8': 'NRC_D8-ONOFF',
    '9': 'NRC_D9-ONOFF',
    '11': 'NRC_D11-ONOFF',
    '12': 'NRC_D12-ONOFF',
    'Tools': 'NRC_PROG-ONOFF',
    'Back': 'NRC_RETURN-ONOFF',
    'Epg': 'NRC_INFO-ONOFF',
    'Exit': 'NRC_CANCEL-ONOFF'
};

var initDeviceInfo = function () {
    log("DEBUG", "Init Device Info");
    try {
        device = JSON.parse(getInfo('currentDevice'));
        log("DEBUG", device.ip + " " + device.port);
        log("DEBUG", "On initDeviceInfo contextId is: " + device.contextId);
        log("DEBUG", "On initDeviceInfo source is: " + getSource(device.contextId));
        if (device.modelNumber) {
            if (device.friendlyName){
                document.getElementById("deviceInfo").innerHTML = `${device.modelNumber} - ${device.friendlyName}`;
            }
            else {
                document.getElementById("deviceInfo").innerHTML = `${device.brandName} - ${device.modelNumber}`;
            }
        }
    } catch (err) {
        log("ERROR", "Unable to initialize device info: " + err);
    }
}

var getInfo = function (value) {
    var info = "empty";
    switch (value) {
        case "currentDevice":
            info = Android.getCurrentDeviceInfo();
            break;
        case "currentActivity":
            info = Android.getCurrentActivityInfo();
            break;
        case "currentRoom":
            info = Android.getCurrentRoomInfo();
            break;
    }
    return info;
}

var setVolume = function (status, volume) {
    log("DEBUG", "setVolume: " + volume);
    if (commandStatus === 'getVolumeUp') {
        volume = parseInt(volume) + 1;
    } else {
        volume = volume > 0 ?  parseInt(volume) - 1 : 0;
    }
    commandStatus = '';
    log("DEBUG", "volume: " + volume);
    submitRequest(URL_RENDERING, URN_RENDERING, ACTION_SETVOLUME, {
        args: "<InstanceID>0</InstanceID><Channel>Master</Channel><DesiredVolume>" + volume + "</DesiredVolume>"
    });
}

var setMute = function(enable){
    log("DEBUG", "mute: " + enable);
    var data = (enable)? 0 : 1;
    commandStatus = '';
    submitRequest(
		URL_RENDERING,
		URN_RENDERING,
		ACTION_SETMUTE,
		{
			args: "<InstanceID>0</InstanceID><Channel>Master</Channel><DesiredMute>"+data+"</DesiredMute>"
		}
	);
} 

var getHttpJsonResponse = function(response) {
    log("DEBUG", "getHttpJsonResponse : " + response);
    var headers, status, result, url, resultIndex, statusindex, resultVal;
    var extractedRespone = response.replace('="', "='");
    extractedRespone = extractedRespone.replace('""', "'\"");
    extractedRespone = extractedRespone.replace('"?', "'?");
    extractedRespone = extractedRespone.replace('"{', "{");
    extractedRespone = extractedRespone.replace('}"', "}");
    extractedRespone = response.replace(/\n/g, '');
    log("DEBUG", "extractedRespone: " + extractedRespone);
    resultIndex = extractedRespone.indexOf("result");
    statusindex =  extractedRespone.indexOf(',"statusCode');
    resultVal = extractedRespone.substring(resultIndex+'result":"'.length, statusindex);
    log("DEBUG", "resultVal: " + resultVal);
    var statusVal = extractedRespone.substring(statusindex).split(":")[1];
    log("DEBUG","statusVal:" + statusVal )
    var status = parseInt(statusVal.substring(0, statusVal.length-1));
    log("DEBUG","status:" + status );
    if (status === 200) {
        log("DEBUG", "commandStatus: " + commandStatus);
        removeCommandTimeout();
        toggleAlertMessage("none");
        switch (commandStatus) {
            case 'getVolumeUp':
            case 'getVolumeDown':
                var regex = /<CurrentVolume>(\d*)<\/CurrentVolume>/gm;
                var match = regex.exec(resultVal);
                log("DEBUG", "match: " + match);
                if (match !== null) {
                    var volume = match[1];
                    setVolume(commandStatus, volume);
                }
                break;
            case 'mute':
                var regex = /<CurrentMute>([0-1])<\/CurrentMute>/gm;
                var match = regex.exec(resultVal);
                log("DEBUG", "match: " + match);
				if(match !== null){
					var mute = (match[1] == '1');
					setMute(mute);
				}
                break;
            default:
                try {
                    sendCommandDetailSentEvent(true, commandVal, device.contextId, getSource(device.contextId));                
                } catch (error) {
                    sendInsightEvent(true, commandVal);                                        
                }
                break;
        }
    }else{
        log("DEBUG", "Command sent Failure");
        toggleAlertMessage("block");
        try {
            sendCommandDetailSentEvent(false, commandVal, device.contextId, getSource(device.contextId));                
        } catch (error) {
            sendInsightEvent(false, commandVal);                                        
        }       
    }
}

var getHttpResponse = function (response) {

}

var removeCommandTimeout = function () {
    if (commandTimeout) {
        clearTimeout(commandTimeout);
        commandTimeout = null;
    }
}

var submitRequest = function (url, urn, action, options) {
    log("DEBUG", `url:${url}, action:${action}, urn: ${urn}, options: ${options['args']}`);
    var command_str = "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n<s:Envelope xmlns:s=\"http://schemas" +
            ".xmlsoap.org/soap/envelope/\" s:encodingStyle=\"http://schemas.xmlsoap.org/soap/" +
            "encoding/\">\n <s:Body>\n  <u:" + action + " xmlns:u=\"urn:" + urn + "\">\n   " + options['args'] + "\n  </u:" + action + ">\n </s:Body>\n</s:Envelope>\n";
    var method = 'POST';
    var headersJsonMap = JSON.stringify({
        'Content-Type': 'text/xml; charset="utf-8"',
        'SOAPACTION': '"urn:'+urn+'#'+action+'"'
    });
    var postUrl = `http://${device.ip}:55000${url}`;
    log("DEBUG", "command_str: "+ command_str);
    log("DEBUG", "headersJsonMap: "+ headersJsonMap);
    removeCommandTimeout();
    commandTimeout = setTimeout(function() {
        log("ERROR", 'commandTimeout');
        toggleAlertMessage('block');
    }, 2000);
    try {
        Android.sendHttpRequest(method, headersJsonMap, command_str, postUrl);
    } catch (error) {
        log("ERROR", "Android Interface not found.")
    }
}

var sendKey = function (command) {
    log("DEBUG", "sendKey: " + command);
    log("DEBUG", "ACTION_SENDKEY: " + ACTION_SENDKEY);
    submitRequest(URL_NETWORK, URN_NETWORK, ACTION_SENDKEY, {
        args: "<" + ARG_SENDKEY + ">" + command + "</" + ARG_SENDKEY + ">"
    });
}

var getVolume = function () {
    submitRequest(URL_RENDERING, URN_RENDERING, ACTION_GETVOLUME, {args: "<InstanceID>0</InstanceID><Channel>Master</Channel>"});
}

var getMuteState = function () {
    submitRequest(URL_RENDERING, URN_RENDERING, ACTION_GETMUTE, {args: "<InstanceID>0</InstanceID><Channel>Master</Channel>"});
}

var sendCommandRequest = function (command) {
    log("DEBUG", "makeSendCommandRequest: " + command);
    log("DEBUG", "makeSendCommandRequest: " + commandToCapability[command]);
    switch (command) {
        case 'Volume_Up':
            commandStatus = 'getVolumeUp';
            getVolume();
            break;
        case 'Volume_Down':
            commandStatus = 'getVolumeDown';
            getVolume();
            break;
        case 'Mute':
            commandStatus = 'mute';
            getMuteState();
            break;
        default:
            sendKey(commandToCapability[command]);
            break;
    }
}

var startTimer = function (event, command) {
    pressTimer = setTimeout(function () {
        removeCommandTimeout();
        sendPanasonicCommand(event, command);
        startTimer(event, command);
    }, 200);
}

var clearTimer = function () {
    clearTimeout(pressTimer);
}

var sendPanasonicCommand = function (event, command) {
    try {
        sendRemoteButtonPressed(device.contextId, "Panasonic", command, getSource(device.contextId));
    } catch (error) {
        log("ERROR", "sendRemoteButtonPressed is not defined");
    }
    highlightingButtons(event, command);
    sendDeviceCommand(command, "inapp");
}

var sendDeviceCommand = function (command, location) {
    commandSentLocation = location;
    commandVal = command;
    sendCommandRequest(command);
}