var deviceControl = {"ip": "192.168.1.109", "port": "8060"};
var device = {"ip": "192.168.1.109", "port": "8060","contextId":151, "source":"inapp"};
var commandTimeout;
var pressTimer;

var commandMap = {
    'Home': "Home",
    'roku_rewind': "Rev",
    'roku_fast_forward': "Fwd",
    'roku_play': "Play",
    'Select': "Select",
    'Navigate_Left': "Left",
    'Navigate_Right': "Right",
    'Navigate_Down': "Down",
    'Navigate_Up': "Up",
    'Back': "Back",
    'InstantReplay': "InstantReplay",
    'Options': "Info",
    'Backspace': "Backspace",
    'Search': "Search",
    'Enter': "Enter",
    'Power':"Power",
    'VolumeMute': "VolumeMute",
    'VolumeDown': "VolumeDown",
    'VolumeUp':"VolumeUp",
    'Mute': "VolumeMute",
    'Volume_Down': "VolumeDown",
    'Volume_Up': "VolumeUp"
    
}

var toggleSearchBox = function(event) {
    event.stopPropagation();
    event.target.parentNode.style.backgroundImage = "url('../lib/base/images/common_btn_press.png')";
    setTimeout(function() {
        event.target.parentNode.style.backgroundImage = "url('../lib/base/images/common_btn_normal.png')";
    }, 50);
    var elem = document.getElementById("search-box");
    if (elem.style.display === "none" || elem.style.display === "") {
        elem.style.display = "block";
        document.getElementById("searchKey").focus();
    } else {
        document.getElementById("searchKey").value = "";
        elem.style.display = "none";
    }
}

var startTimer = function(event, command) {
    pressTimer = setTimeout(function() {
        removeCommandTimeout();
        sendRokuCommand(event, command);
        startTimer(event, command);
    }, 100);
}

var clearTimer = function() {
    clearTimeout(pressTimer);
}

var removeCommandTimeout = function(){
    if (commandTimeout) {
        clearTimeout(commandTimeout);
        commandTimeout = null;
    }
    toggleAlertMessage("none");
}

var makeRequest = function(endPoint, command) {
    removeCommandTimeout();
    commandTimeout = setTimeout(function(){
        toggleAlertMessage("block");
    }, 2000)
    fetch(endPoint, {
        method: 'POST',
        mode: 'no-cors',
        headers: {
            'Content-Type': 'application/x-www-form-urlencoded'
        }
    }).then(function(res) {
        removeCommandTimeout();
        toggleAlertMessage("none");
        log("DEBUG", "command sent successfully: " + command);
		try {
				sendCommandDetailSentEvent(true, command, device.contextId, getSource(device.contextId));
			} catch (error) {
				sendInsightEvent(true, command);                    
			}
    }).catch(function(error) {
        removeCommandTimeout();
        toggleAlertMessage("block");
        log("ERROR", "Error While sending command : " + error);
        try {
				sendCommandDetailSentEvent(false, command, device.contextId, getSource(device.contextId));
			} catch (error) {
				sendInsightEvent(false, command);                    
			}
    })
}

var sendRokuCommand = function(event, command) {

    try {
        sendRemoteButtonPressed(device.contextId, "Roku", command, getSource(device.contextId));
    } catch (error) {
        log("ERROR", "sendRemoteButtonPressed is not defined");
    }
    highlightingButtons(event, command);
    //sendCommand(command);
    sendDeviceCommand(command, getSource(device.contextId));
}

function sendCommand(command) {
    var rokuCommand = commandMap[command];
    if (!rokuCommand) {
        rokuCommand = command;
    }
    var endPoint = `http://${deviceControl.ip}:${deviceControl.port}/keypress/${rokuCommand}`;
    makeRequest(endPoint, rokuCommand);
    log("DEBUG", "sendCommand endPoint: " + endPoint);
}


var sendDeviceCommand = function (command, location) {
    var rokuCommand = commandMap[command];
    if (!rokuCommand) {
        rokuCommand = command;
    }
    var endPoint = `http://${deviceControl.ip}:${deviceControl.port}/keypress/${rokuCommand}`;
    makeRequest(endPoint, rokuCommand);
    log("DEBUG", "sendCommand endPoint: " + endPoint);
}

function search(event) {
    var searchKey = document.getElementById("searchKey").value;
    setTimeout(function() {
        var endPoint = "http://" + deviceControl.ip + ":" + deviceControl.port + "/search/browse?keyword=" + searchKey;
        makeRequest(endPoint, "search keyword: " + searchKey);
    }, 500)
}

function xmlToJson(xml) {

    // Create the return object
    console.log("XMl:" + xml);
    var obj = {};

    if (xml.nodeType == 1) { // element
        // do attributes
        if (xml.attributes.length > 0) {
            obj["attributes"] = {};
            for (var j = 0; j < xml.attributes.length; j++) {
                var attribute = xml.attributes.item(j);
                obj["attributes"][attribute.nodeName] = attribute.nodeValue;
            }
        }
    } else if (xml.nodeType == 3) { // text
        obj = xml.nodeValue;
    }

    // do children
    if (xml.hasChildNodes()) {
        for (var i = 0; i < xml.childNodes.length; i++) {
            var item = xml.childNodes.item(i);
            var nodeName = item.nodeName;
            if (typeof(obj[nodeName]) == "undefined") {
                obj[nodeName] = xmlToJson(item);
            } else {
                if (typeof(obj[nodeName].push) == "undefined") {
                    var old = obj[nodeName];
                    obj[nodeName] = [];
                    obj[nodeName].push(old);
                }
                obj[nodeName].push(xmlToJson(item));
            }
        }
    }
    return obj;
};

var getAppsOnRoku = function(device) {
    var endPoint = `http://${device.ip}:${device.port}/query/apps`;
    try {
        var method = "GET";
        var jsonPost = JSON.stringify({});
        var headersJsonMap = JSON.stringify({});
        var url = `http://${device.ip}:${device.port}/query/apps`;
        Android.sendHttpRequest(method, headersJsonMap, jsonPost, url);
    } catch (err) {
        log("ERROR", "sendHttpRequest: No Android Interface:" + err);
    }
}

var launchApp = function(appID) {
    var endPoint = `http://${device.ip}:${device.port}/launch/${appID}`;
    makeRequest(endPoint, "Application Launched: " + appID);
}

var buildAppsHtml = function(apps) {
    var appListitemsHtml = '';
    for (var i = 0; i < apps.length; i++) {
        console.log()
        if (apps[i].attributes.type === 'appl') {
            appListitemsHtml += `<span class="app-item" onclick="launchApp(${apps[i].attributes.id})"><img src="http://${device.ip}:${device.port}/query/icon/${apps[i].attributes.id}" width="80px"></span>`;
        }
    }

    return appListitemsHtml;
}

function getHttpJsonResponse(response) {
    var extractedRespone = response.replace('""', "'\"").replace('"?', "'?").replace('"{', "{").replace('}"', "}").replace(/\n/g, '');
    log("DEBUG", "extractedRespone: " + extractedRespone);
    extractedRespone = JSON.parse(extractedRespone);
    var result = extractedRespone.result;
    log("DEBUG", "apps result: " + result);
    var appsList = xmlToJson(result);
    var apps = appsList.apps.app;
    document.getElementById("appList").innerHTML = buildAppsHtml(apps);
}

function initDeviceInfo() {
    try {
        device = JSON.parse(getInfo('currentDevice'));
        deviceControl = device;
        log("DEBUG", "initDeviceInfo ip and port: " + device.ip + " " + device.port);
        if (device.modelNumber) {
            if (device.friendlyName){
                document.getElementById("deviceInfo").innerHTML = `${device.modelNumber} - ${device.friendlyName}`;
            }
            else {
                document.getElementById("deviceInfo").innerHTML = `${device.brandName} - ${device.modelNumber}`;
            }
        }
        // getAppsOnRoku(device);
    } catch (err) {
        log("ERROR", "Unable to initialize device info: " + err);
    }
}

function getInfo(value) {
    var info = "empty";
    switch (value) {
        case "currentDevice":
            info = Android.getCurrentDeviceInfo();
            break;
        case "currentActivity":
            info = Android.getCurrentActivityInfo();
            break;
        case "currentRoom":
            info = Android.getCurrentRoomInfo();
            break;
    }
    return info;
}