'use strict';

// #### Mapping Peel To Samsung Commands
var commandMap = {
    'Power_off': 'KEY_POWEROFF',
    'Power': 'KEY_POWER',
    'Volume_Up': 'KEY_VOLUP',
    'Volume_Down': 'KEY_VOLDOWN',
    'Mute': 'KEY_MUTE',
    'Input': 'KEY_SOURCE',
    'Pause': 'KEY_PAUSE',
    'Play': 'KEY_PLAY',
    'Skip_Back': 'KEY_PREV',
    'Skip_Forward': 'KEY_NEXT',
    'Rewind': 'KEY_REWIND',
    'Fast_Forward': 'KEY_FF',
    'Record': 'KEY_REC',
    'Menu': 'KEY_MENU',
    'Home': 'KEY_CONTENTS',
    'Back': '',
    'Info': 'KEY_INFO',
    'Exit': '',
    'Options': '',
    'LiveTV': 'KEY_TV',
    'DVR': 'KEY_DVR',
    'Red': 'KEY_RED',
    'Green': 'KEY_GREEN',
    'Blue': 'KEY_CYAN',
    'Yellow': 'KEY_YELLOW',
    'Select': 'KEY_ENTER',
    'Previous': '',
    'Channel_Up': 'KEY_CHUP',
    'Channel_Down': 'KEY_CHDOWN',
    'Navigate_Left': 'KEY_LEFT',
    'Navigate_Right': 'KEY_RIGHT',
    'Navigate_Up': 'KEY_UP',
    'Navigate_Down': 'KEY_DOWN',
    'Dot_DASh': 'KEY_PLUS100',
    'Stop': 'KEY_STOP',
    '0': 'KEY_0',
    '1': 'KEY_1',
    '2': 'KEY_2',
    '3': 'KEY_3',
    '4': 'KEY_4',
    '5': 'KEY_5',
    '6': 'KEY_6',
    '7': 'KEY_7',
    '8': 'KEY_8',
    '9': 'KEY_9',
    '11': 'KEY_11',
    '12': 'KEY_12',
    'Tools': 'KEY_TOOLS',
    'CHList': '',
    'AUDIO': '',
    'Anynet': 'KEY_ANYNET',
    'Antenna': 'KEY_ANTENA',
    'HDMI': 'KEY_HDMI',
    'Svideo1': 'KEY_SVIDEO1',
    'Component1': 'KEY_COMPONENT1',
    'Component2': 'KEY_COMPONENT2',
    'DVI': 'KEY_DVI',
    'AV1': 'KEY_AV1',
    'Epg': 'KEY_GUIDE',
};
var socket;
var identity;
var pinPort = 8080;
var pairingTimeoutValue = 120000;
var pairingTimeout;
var DUID;
var device = {
    "ip": "192.168.1.38",
    "port": "8080",	 
	"contextId":151, 
    "source":"inapp"
};
var pressTimer;
var pairingText = '<span style="font-size: 24px;">Pairing...</span>';
var isPaired = false;
var isConnected = false;
var commandVal;
var pairingCommandTimeout;
var connectionTimeout;
var isPairingTriggeredFromLockPanel = false;
var commandSentLocation;

var checkIsPairingDone = function(deviceIp) {
    if (document.getElementById("pairingValue")) {
        document.getElementById("pairingValue").value = '';
    }
    device = JSON.parse(getInfo('currentDevice'));
    var storedPairingCode = localStorage.getItem(device.ip);
    log("DEBUG", "storedPairingCode : " + storedPairingCode);
    console.log(storedPairingCode);
    if (storedPairingCode !== null || typeof storedPairingCode === "undefined" || storedPairingCode === '') {
        log("DEBUG", "identity: " + storedPairingCode);
        identity = JSON.parse(storedPairingCode);
        isPaired = true;
        wsConnect();
    } else {
        sendPairingTriggredEvent();
        showPin();
    }
}

var getRequestUri = function(step) {
    var appId = '12345';
    var deviceId = '74f3606e-1ce4-45ee-a800-40f2271949d0';
    return '/ws/pairing?step=' + step + '&app_id=' + appId + '&device_id=' + deviceId;
}

var buildEmitMessage = function(name, payload) {
    log("DEBUG", "buildEmitMessage: " + name + ",payload : " + JSON.stringify(payload));
    return '5::/com.samsung.companion:' + JSON.stringify({
        name: name,
        args: [
            payload
        ]
    })
}

var startTimer = function(event, command) {
    pressTimer = setTimeout(function() {
        sendSamsungCommand(event, command);
        startTimer(event, command);
    }, 200);
}

var clearTimer = function() {
    clearTimeout(pressTimer);
}

var sendSamsungCommand = function(event, command, type) {
    highlightingButtons(event, command);
    sendDeviceCommand(command, "inApp");
}

var checkLocalStoredData = function(storedData, commandSentLocation) {
    if (storedData) {
        try {
            storedData = JSON.parse(storedData);
            DUID = storedData.duid;
        } catch (e) {
            log("DEBUG", "unable to parse JSON 183");
        }
    }
}

var checkToOpenDeviceRemoteInApp = function(command, commandSentLocation) {
    try {
        var storedData = localStorage.getItem(device.ip);
        log("DEBUG", "storedData: " + storedData);
        if (!storedData) {
            storedData = {};
            log("DEBUG", "storedData is null");
            storedData.commandSentLocation = commandSentLocation;
            storedData.commandSent = command;
            localStorage.setItem(device.ip, JSON.stringify(storedData));
            commandSentLocation = '';
            Android.openDeviceRemote(device.id);
            log("DEBUG", "openDeviceRemoteSent");
            return true;
        }
        log("DEBUG", "openDeviceRemoteSent");
    } catch (e) {
        log("DEBUG", "Android interface not found")
    }
    return false;
}

var sendDeviceCommand = function(command, location) {
    commandSentLocation = location;
    log("DEBUG", DUID + ", commandMap : " + commandMap[command]);
    commandVal = command;
    if (location === 'widget') {
        if(checkToOpenDeviceRemoteInApp(command, location)){
            sendPairingEvent(device.type, device.brandName, "TRIGGERED", "WIDGET");
            return;
        }
        log("DEBUG", "checking recursive");
    }
    var storedData = localStorage.getItem(device.ip);
    checkLocalStoredData(storedData, location);

    if (!isPaired || !isConnected || !DUID) {
        checkIsPairingDone();
        return;
    }

    removeCommandTimeout();

    pairingCommandTimeout = setTimeout(function() {
        log("ERROR", 'pairingCommandTimeout');
        isConnected = false;
        //sendInsightEvent(false, commandVal);			
        try {
            sendCommandDetailSentEvent(false, commandVal, device.contextId, getSource(device.contextId));
        } catch (errors) {
            sendInsightEvent(false, commandVal);                    
        }
        toggleAlertMessage('block');

    }, 2000);
    log("DEBUG", "device is paired and connected");

    //2015 model TVs
    var samsungTizenpattern = new RegExp('[U|E|N]+\\d+[J]([S|U|No|P])?\\d+([W|K|B|U|T])?');
    //2014 model TVs
    var samsungSakepPattern = new RegExp('[U|E|N]+\\d+[H]([S|U|No|P])?\\d+([W|K|B|U|T])?');
    if (device.modelNumber) {
        if (command === 'Power') {
            if (samsungTizenpattern.test(device.modelNumber)) {
                command = 'Power';
            } else if (samsungSakepPattern.test(device.modelNumber)) {
                command = 'Power_off';
            }
        }
    }

    socket.send(buildEmitMessage(
        'callCommon',
        encryptData(identity.aesKey, identity.sessionId, {
            method: 'POST',
            body: {
                plugin: 'RemoteControl',
                version: '1.000',
                api: 'SendRemoteKey',
                param1: DUID,
                param2: 'Click',
                param3: commandMap[command],
                param4: 'false'
            }
        })), { mask: true });
}

var openCheckPopup = function(heading, info, success, warning) {
    log("DEBUG", "OPEN CHECK TV POPUP");
    document.getElementById("messageHeading").innerHTML = heading;
    document.getElementById("messageInfo").innerHTML = info;
    document.getElementById("warningButton").innerHTML = warning;
    document.getElementById("successButton").innerHTML = success;
    document.getElementById('pairingInputContainer').style.display = 'none';
    document.getElementById("loader-container").style.display = "none";
    document.getElementById("pairingCodeBox").style.display = 'block';
    document.getElementById('tvPopOn').style.display = 'block';
}

var handleEvent = function(aesKey, sessionId, event) {
    log("DEBUG", "Handle Event");
    if (event.name === 'receiveCommon') {
        try {
            log("DEBUG", "receiveCommon: " + event.args);
            var decrypted = JSON.parse(decryptData(aesKey, event.args))
            log("DEBUG", "decrypted: " + JSON.stringify(decrypted));
            if (decrypted.plugin === 'NNavi' && decrypted.api === 'GetDUID') {
                DUID = decrypted.result;
                try {
                    var storedData = JSON.parse(localStorage.getItem(device.ip));
                    storedData.duid = DUID;
                    localStorage.setItem(device.ip, JSON.stringify(storedData));
                } catch (e) {
                    log("DEBUG", "unable to parse JSON");
                }

                log("DEBUG", "GetDUID: " + decrypted.result + ":" + decrypted.api);
                // self.connected()
                if (document.getElementById("pairingCodeBox")) {
                    document.getElementById("loader-container").style.display = 'none';
                    document.getElementById("pairingCodeBox").style.display = 'none';
                }
            }else if(decrypted.plugin === 'RemoteControl' && decrypted.api === 'SendRemoteKey'){
                try {
                    sendCommandDetailSentEvent(true, commandVal, device.contextId, getSource(device.contextId));
                } catch (errors) {
                    sendInsightEvent(true, commandVal);                    
                }
				//sendInsightEvent(true, commandVal);
				
            }
        } catch (error) {
            //sendInsightEvent(false, commandVal);				
            try {
                sendCommandDetailSentEvent(false, commandVal, device.contextId, getSource(device.contextId));
            } catch (errors) {
                sendInsightEvent(false, commandVal);                    
            }
            log("ERROR", 'handleEvent : unable to decrypt data');
            isPaired = false;
            sendPairingTriggredEvent();
            showPin();
        }
    }
}

var disconnect = function(status) {
    log("DEBUG", "disconnect: " + status);
    commandVal = '';
    removeCommandTimeout();
    removePairingTimeout();
    removeConnectionTimeout();
    if (document.getElementById("pairingCodeBox")) {
        document.getElementById("pairingCodeBox").style.display = "none";
    }
    switch (status) {
        case "off":
            toggleAlertMessage("block");
            break;
        case "connect":
            toggleAlertMessage("block");
            break;
        case "pair":
            openCheckPopup("", "PAIRING FAILED", "TRY AGAIN", "CANCEL");
            // checkIsPairingDone();
            break;
    }
    /**
     * Shut down the socket if it's open
     */
    if (socket) {
        socket.close()
        socket = null
    }

}

var removePairingTimeout = function() {
    if (pairingTimeout) {
        clearTimeout(pairingTimeout)
        pairingTimeout = null;
    }
}

var removeConnectionTimeout = function() {
    if (connectionTimeout) {
        clearTimeout(connectionTimeout);
        connectionTimeout = null;
    }
}

var removeCommandTimeout = function() {
    if (pairingCommandTimeout) {
        clearTimeout(pairingCommandTimeout);
        pairingCommandTimeout = null;
    }
}

var onClose = function(event) {
    log('ERROR', 'WS_CLOSE: ' + event.reason ? event.reason : event);
    isConnected = false;
    disconnect("connect");
}

var onError = function(event) {
    var reason = event.reason ? event.reason : event
    reason = JSON.stringify(reason);
    console.log(reason);
    log('ERROR', 'WS_ERROR :' + reason);
    isConnected = false;
    disconnect("connect")
}

var onMessage = function(event) {
    var data = event.data;
    isConnected = true;
    log("DEBUG", "onmessage: " + data);

    // mimicking socket.io in pure websockets
    if (data === '1::') {
        socket.send('1::/com.samsung.companion', { masked: true })
        log("DEBUG", "SOCKET MESSAGE:  1::/com.samsung.companion");
        socket.send(buildEmitMessage(
            'registerPush',
            encryptData(identity.aesKey, identity.sessionId, {
                eventType: 'EMP',
                plugin: 'SecondTV'
            })), { mask: true })

        socket.send(buildEmitMessage('registerPush',
            encryptData(identity.aesKey, identity.sessionId, {
                eventType: 'EMP',
                plugin: 'RemoteControl'
            })), { mask: true })

        socket.send(buildEmitMessage('callCommon',
            encryptData(identity.aesKey, identity.sessionId, {
                method: 'POST',
                body: {
                    plugin: 'NNavi',
                    api: 'GetDUID',
                    version: '1.000'
                }
            })), { mask: true });
        if (commandVal) {
            log("DEBUG", "commandVal: " + commandVal);
            sendDeviceCommand(commandVal, "inApp");
        }
    } else if (data.startsWith('5::/com.samsung.companion:')) {
        log("DEBUG", "SOCKET MESSAGE:  5::/com.samsung.companion:");
        removeCommandTimeout()
        toggleAlertMessage("none");
        log("DEBUG", "command Sent: " + commandMap[commandVal]);
        handleEvent(identity.aesKey, identity.sessionId, JSON.parse(data.slice('5::/com.samsung.companion:'.length)));
    } else if (data === '2::') {
        // keep-alive
        log("DEBUG", "keep-alive: " + data);
        socket.send('2::', { masked: false })
    } else {
        log('DEBUG', 'unknown message received: ' + data)
    }
}

var onOpen = function(event) {
    log('DEBUG', 'WS_OPEN');
    removeCommandTimeout();
}

var connectToSocket = function() {
    fetch('http://' + device.ip + ':8000/common/1.0.0/service/startService?appID=com.samsung.companion').then(function() {
        return fetch('http://' + device.ip + ':8000/socket.io/1').then(function(res) {
            log("DEBUG", "wsConnect inside");
            return res.text().then(function(body) {
                var handshake = body.split(':');
                log("DEBUG", "handshake: " + handshake[0]);
                return handshake[0]
            });
        }).then(function(mask) {
            removeConnectionTimeout();
            // TRY to USE WebSockets (http://developer.mozilla.org/en-US/docs/Web/API/WebSockets_API/Writing_WebSocket_client_applications). if this didn't work try using (http://socket.io/)
            log("DEBUG", "mask: " + mask);
            socket = new WebSocket('ws://' + device.ip + ':8000/socket.io/1/websocket/' + mask);

            if (socket) {
                socket.onclose = onClose
                socket.onerror = onError
                socket.onmessage = onMessage
                socket.onopen = onOpen
            } else {
                log('DEBUG', 'unable to connect to device.');
                //sendInsightEvent(false, commandVal);
                try {
                    sendCommandDetailSentEvent(false, commandVal, device.contextId, getSource(device.contextId));
                } catch (errors) {
                    sendInsightEvent(false, commandVal);                    
                }
                disconnect("off");
            }
        })
    }).catch(function(err) {
        log("ERROR", err);
        isConnected = false;
        //sendInsightEvent(false, commandVal);
        try {
            sendCommandDetailSentEvent(false, commandVal, device.contextId, getSource(device.contextId));
        } catch (errors) {
            sendInsightEvent(false, commandVal);                    
        }
        disconnect("off");
    });
}

var sendPairingTriggredEvent = function(){
    if(!isPairingTriggeredFromLockPanel){
        sendPairingEvent(device.type, device.brandName, "TRIGGERED", "CONTROL_PAD");                    
    }
}

var wsConnect = function() {
    log("DEBUG", "identity : " + identity);
    log("DEBUG", "deviceip: " + device.ip);
    if (!device) {
        device = JSON.parse(getInfo('currentDevice'));
    }
    var storedPairingCode = localStorage.getItem(device.ip);
    if (storedPairingCode !== null || typeof storedPairingCode === "undefined" || storedPairingCode === '') {
        identity = JSON.parse(storedPairingCode);
        isPaired = true;
    } else {
        sendPairingTriggredEvent();
        showPin();
        return;
    }
    removeConnectionTimeout();
    connectionTimeout = setTimeout(function() {
        toggleAlertMessage("block");
    }, 2000);
    connectToSocket();
}

var pairTV = function() {
    document.getElementById("error").innerHTML = "";
    var pairingKey = document.getElementById("pairingValue").value;
    pairingKey = pairingKey.trim();
    log("DEBUG", "pairingKey: " + pairingKey);
    if (pairingKey === '' || typeof pairingKey === 'undefined') {
        //TODO: SHOW ERROR
        document.getElementById("error").innerHTML = "Enter valid pin";
        return;
    }
    document.getElementById('pairingInputContainer').style.display = 'none';
    document.getElementById('tvPopOn').style.display = 'none';
    document.getElementById('loading-content').innerHTML = pairingText;
    document.getElementById('loader-container').style.display = 'block';
    sendPairingEvent(device.type, device.brandName, "STARTED", "CONTROL_PAD");
    helloExchange(pairingKey);
}

var showPinPopUp = function() {
    setTimeout(function(){
        document.getElementById("loader-container").style.display = "none";
        document.getElementById('tvPopOn').style.display = 'none';
        document.getElementById('error').innerHTML = '';
        document.getElementById('pairingInputContainer').style.display = 'block';
        sendPairingEvent(device.type, device.brandName, "SHOWN", "CONTROL_PAD");
    }, 100); 
}

var showLoader = function() {
    document.getElementById('pairingInputContainer').style.display = 'none';
    document.getElementById('tvPopOn').style.display = 'none';
    document.getElementById('loading-content').innerHTML = pairingText;
    document.getElementById('loading-foot-content').style.display = 'none';
    document.getElementById("loader-container").style.display = "block";
    document.getElementById('pairingCodeBox').style.display = 'block';
}

var cancel = function(event) {
    event.stopPropagation();
    document.getElementById("tvPopOn").style.display = 'none';
    document.getElementById("successButton").innerHTML = '';
    document.getElementById("warningButton").innerHTML = '';
    document.getElementById("pairingInputContainer").style.display = 'none';
    document.getElementById("loader-container").style.display = 'none';
    document.getElementById("pairingCodeBox").style.display = 'none';
    toggleAlertMessage("none");
}

var success = function(event) {
    document.getElementById("tvPopOn").style.display = 'none';
    document.getElementById("successButton").innerHTML = '';
    document.getElementById("warningButton").innerHTML = '';
    document.getElementById("pairingInputContainer").style.display = 'none';
    document.getElementById("loader-container").style.display = 'none';
    document.getElementById("pairingCodeBox").style.display = 'none';
    toggleAlertMessage("none");
    checkIsPairingDone();
}

var showPin = function() {
    showLoader();
    removePairingTimeout();

    pairingTimeout = setTimeout(function() {
        log("ERROR", 'pairing timeout');
        isConnected = false;
        sendPairingResultEvent(false, device.type, device.brandName);
        disconnect("pair");
    }, pairingTimeoutValue);

    var headers = new Headers();
    var initConfig = {
        method: 'POST',
        // headers: headers,
        mode: 'no-cors',
        cache: 'default'
    };

    fetch('http://' + device.ip + ':' + pinPort + '/ws/apps/CloudPINPage', initConfig).then(function(res) {
        return res.text()
    }).then(function(text, status) {
        var match = /<state[^>]*>([\s\S]*?)<\/state>/.exec(text)
        if (match && match[1]) {
            return match[1].toUpperCase() !== 'STOPPED'
        }

        return true
    }).then(function(pinShowing) {
        if (!pinShowing) {
            return fetch('http://' + device.ip + ':' + pinPort + '/ws/apps/CloudPINPage', {
                method: 'POST',
                body: 'pin4',
                mode: 'no-cors',
                headers: {
                    'Content-Type': 'application/json'
                }
            }).then(function(res) {
                return res.ok
            })
        } else {
            return true
        }
    }).then(function(pinShowing) {
        return fetch('http://' + device.ip + ':' + pinPort + getRequestUri(0) + '&type=1', {
            mode: 'no-cors',
        }).then(function(response) {
            showPinPopUp();
        })
    }).catch(function(err) {
        isConnected = false;
        sendPairingResultEvent(false, device.type, device.brandName);
        disconnect("connect");
        log("ERROR", err);
    });

}

var hidePin = function() {
    return fetch('http://' + device.ip + ':' + pinPort + '/ws/apps/CloudPINPage/run', {
        method: 'DELETE'
    });
}

var helloExchange = function(pin) {
    if (pin !== '') {
        // constants used for pairing
        var userId = '654321';
        var serverHello = generateServerHello(userId, pin)
        log("DEBUG", 'serverHello: ' + serverHello);
        try {
            var method = "POST";
            var payload = JSON.stringify({
                'auth_Data': {
                    'auth_type': 'SPC',
                    'GeneratorServerHello': serverHello
                }
            });
            var url = 'http://' + device.ip + ':' + pinPort + getRequestUri(1);
            var contentType = "application/json";
            var headersJsonMap = JSON.stringify({ "Content-Type": contentType });
            Android.sendHttpRequest(method, headersJsonMap, payload, url);
        } catch (err) {
            log("ERROR", "sendHttpRequest: No Android Interface:" + err);
        }
    }
}

var acknowledgeExchange = function(requestId) {
    var serverAck = generateServerAcknowledge()
    log("DEBUG", `generatedServerAcknowledge: ${serverAck}`);

    try {
        var method = "POST";
        var userAgent = "";
        var payload = JSON.stringify({
            'auth_Data': {
                'auth_type': 'SPC',
                'request_id': requestId,
                'ServerAckMsg': serverAck
            }
        });
        var url = 'http://' + device.ip + ':' + pinPort + getRequestUri(2);
        var contentType = "application/json";
        var headersJsonMap = JSON.stringify({ "Content-Type": contentType });
        Android.sendHttpRequest(method, headersJsonMap, payload, url);
    } catch (err) {
        log("ERROR", "sendHttpRequest: No Android Interface:" + err);
    }
}

function log(level, msg) {
    try {
        //For PSR only
        Android.log(level, msg);
    } catch (err) {
        console.log(msg);
    }
}

function initDeviceInfo() {
    console.log("Init Device Info");
    try {
        device = JSON.parse(getInfo('currentDevice'));
        log("DEBUG", JSON.stringify(device));
        if (device.modelNumber) {
            if (device.friendlyName){
                document.getElementById("deviceInfo").innerHTML = `${device.modelNumber} - ${device.friendlyName}`;
            }
            else {
                document.getElementById("deviceInfo").innerHTML = `${device.brandName} - ${device.modelNumber}`;
            }
        }
        var localStoredData = localStorage.getItem(device.ip);
        log("DEBUG", "localStoredData: " + localStoredData);
        if (localStoredData) {
            try {
                localStoredData = JSON.parse(localStoredData);
                log("DEBUG", "There is localStoredData");
                log("DEBUG", "commandSentLocation : " + localStoredData.commandSentLocation);
                if (localStoredData.commandSentLocation && localStoredData.commandSentLocation === 'widget') {
                    let commandSent = localStoredData.commandSent;
                    log("DEBUG", "commandSentLocation : " + commandSent);
                    localStorage.removeItem(device.ip);
                    isPairingTriggeredFromLockPanel = true;
                    sendDeviceCommand(commandSent, "inApp");
                }
            } catch (e) {
                log("DEBUG", "unable to parse JSON");
            }
        }
    } catch (err) {
        log("ERROR", "Unable to initialize device info: " + err);
    }
}

function getInfo(value) {
    var info = "empty";
    switch (value) {
        case "currentDevice":
            info = Android.getCurrentDeviceInfo();
            break;
        case "currentActivity":
            info = Android.getCurrentActivityInfo();
            break;
        case "currentRoom":
            info = Android.getCurrentRoomInfo();
            break;
    }
    return info;
}

function getHttpJsonResponse(response) {
    log("DEBUG", "encryptedDAta : " + response);

    if (typeof response === "undefined" || response === 'null') {
        log("DEBUG", `${url}'s result is ${result}`);
        return;
    }
    //  #### TODO: this is a hardcoded way of doing things, Need to think of a dynamic solution

    if (response.indexOf("step=1") !== -1) {
        var index = response.indexOf("result") + "result".length + 3;
        var statusIndex = response.indexOf("statusCode") - 3;
        // var result = response.substring(329, response.length - 21);
        var result = response.substring(index, statusIndex);
        result = result.trim();
        log("DEBUG", result);
        // result = result.replace("\\", '');
        log("DEBUG", "replaced result: " + result);

        try {
            var parsed = JSON.parse(result);
        } catch (error) {
            log("ERROR", 'unable to parse Json');
        }
        var dataParsed = JSON.parse(parsed.auth_data);
        log("DEBUG", "parsed result: " + JSON.stringify(parsed));
        if (parseClientHello(dataParsed.GeneratorClientHello) === 0) {
            log("DEBUG", "hardcoded process worked for now");
            acknowledgeExchange(dataParsed.request_id);
        } else {
            document.getElementById("loader-container").style.display = "none";
            document.getElementById('tvPopOn').style.display = 'none';
            document.getElementById('error').innerHTML = '';
            document.getElementById('pairingInputContainer').style.display = 'block';
            document.getElementById("error").innerHTML = "Invalid PIN Entered";
            sendPairingResultEvent(false, device.type, device.brandName);
            log("ERROR", '${url} Invalid PIN Entered');
        }
    } else if (response.indexOf("step=2") !== -1) {
        var index = response.indexOf("result") + "result".length + 3;
        var statusIndex = response.indexOf("statusCode") - 3;
        // var result = response.substring(329, response.length - 21);
        var result = response.substring(index, statusIndex);
        // var result = response.substring(329, response.length - 21);
        result = result.trim();
        try {
            var parsed = JSON.parse(result);
        } catch (error) {
            log("ERROR", 'unable to parse Json');
        }
        log("DEBUG", result);
        var dataParsed = JSON.parse(parsed.auth_data);
        log("DEBUG", "ClientAckMsg : " + dataParsed.ClientAckMsg);
        log("DEBUG", "parseClientAcknowledge: " + parseClientAcknowledge(dataParsed.ClientAckMsg).toString());
        if (parseClientAcknowledge(dataParsed.ClientAckMsg)) {
            removePairingTimeout();
            hidePin().then(function() {
                identity = {
                    sessionId: dataParsed.session_id,
                    aesKey: getKey(),
                };
                log("DEBUG", "identity: " + JSON.stringify(identity));
                localStorage.setItem(device.ip, JSON.stringify(identity));
                document.getElementById('loader-container').style.display = 'none';
                isPaired = true;
                sendPairingResultEvent(true, device.type, device.brandName);
                wsConnect();
            });
        } else {
            sendPairingResultEvent(false, device.type, device.brandName);
            log("ERROR", '${url} unable to parseClientAcknowledge');
        }
    }
}
