'use strict';

var socket;
var device = {
    "ip": "192.168.1.38",
    "port": "8001",	 
	"contextId":151, 
    "source":"inapp"
};
var socket;
var pressTimer;
var commandVal = '';
var commandTimeout;

var CONNECT = 'ms.channel.connect';
var DENIED = 'ms.channel.unauthorized';
var TIMEOUT = 'ms.channel.timeOut';

var isConnected = false;

var commandMap = {
    'Power': 'KEY_POWER',
    'Volume_Up': 'KEY_VOLUP',
    'Volume_Down': 'KEY_VOLDOWN',
    'Mute': 'KEY_MUTE',
    'Input': 'KEY_SOURCE',
    'Pause': 'KEY_PAUSE',
    'Play': 'KEY_PLAY',
    'Skip_Back': 'KEY_PREV',
    'Skip_Forward': 'KEY_NEXT',
    'Rewind': 'KEY_REWIND',
    'Fast_Forward': 'KEY_FF',
    'Record': 'KEY_REC',
    'Menu': 'KEY_MENU',
    'Home': 'KEY_CONTENTS',
    'Back': '',
    'Info': 'KEY_INFO',
    'Exit': '',
    'Options': '',
    'LiveTV': 'KEY_TV',
    'DVR': 'KEY_DVR',
    'Red': 'KEY_RED',
    'Green': 'KEY_GREEN',
    'Blue': 'KEY_CYAN',
    'Yellow': 'KEY_YELLOW',
    'Select': 'KEY_ENTER',
    'Previous': '',
    'Channel_Up': 'KEY_CHUP',
    'Channel_Down': 'KEY_CHDOWN',
    'Navigate_Left': 'KEY_LEFT',
    'Navigate_Right': 'KEY_RIGHT',
    'Navigate_Up': 'KEY_UP',
    'Navigate_Down': 'KEY_DOWN',
    'Dot_DASh': 'KEY_PLUS100',
    'Stop': 'KEY_STOP',
    '0': 'KEY_0',
    '1': 'KEY_1',
    '2': 'KEY_2',
    '3': 'KEY_3',
    '4': 'KEY_4',
    '5': 'KEY_5',
    '6': 'KEY_6',
    '7': 'KEY_7',
    '8': 'KEY_8',
    '9': 'KEY_9',
    '11': 'KEY_11',
    '12': 'KEY_12',
    'Tools': 'KEY_TOOLS',
    'CHList': '',
    'AUDIO': '',
    'Anynet': 'KEY_ANYNET',
    'Antenna': 'KEY_ANTENA',
    'HDMI': 'KEY_HDMI',
    'Svideo1': 'KEY_SVIDEO1',
    'Component1': 'KEY_COMPONENT1',
    'Component2': 'KEY_COMPONENT2',
    'DVI': 'KEY_DVI',
    'AV1': 'KEY_AV1',
    'Epg': 'KEY_GUIDE',
};

var base64Encode = function(string) {
    return window.btoa(string);
}

var onOpen = function(event) {
    log('DEBUG', 'WS_OPEN');
}

var connected = function() {
    log('DEBUG', 'WS_CONNECT');
    log("DEBUG", "Device: " + JSON.stringify(device));
    removeCommandTimeout();
    isConnected = true;
    toggleAlertMessage("none");
    sendDeviceCommand(commandVal, device.source);
}

var onMessage = function(event) {
    var json = JSON.parse(event.data);
    log("DEBUG", JSON.parse(event.data));
    switch (json.event) {
        case CONNECT:
            connected()
            break
        case DENIED:
            disconnect();
            break
        case TIMEOUT:
            disconnect();
            break
        case "ed.edenTV.update":
            break;
        default:
            break;
    }
}

var onError = function(event) {
    disconnect(event.reason);
}

var onClose = function(event) {
    disconnect()
}

var booloverlay = function(state) {
    if (document.getElementById("pairingCodeBox")) {
        document.getElementById("loader-container").style.display = state;
        document.getElementById("pairingCodeBox").style.display = state;
    }

}

/**
 * Triggered by a connect call, this will start monitoring the service.
 */
var wsConnect = function() {
    // inspect the initialized service for the correct endpoints
    // booloverlay("block");
    socket = new WebSocket('ws://' + device.ip +
        ':8001/api/v2/channels/samsung.remote.control?name=' + base64Encode('Peel Smart Remote'), 'basic')

    if (socket) {
        socket.onclose = onClose
        socket.onerror = onError
        socket.onmessage = onMessage
        socket.onopen = onOpen
    } else {
        log('DEBUG', 'unable to connect to device.');
        disconnect('unable to connect to device.')
    }
    // is_api_active();
}

var disconnect = function(reason) {
    // #### Shut down the socket if it's open
    isConnected = false;
    if (socket) {
        socket.close()
        socket = null
    }
    toggleAlertMessage("block");
}

// #### Splits a channel into separate number commands, and tunes in to it

var channelSet = function(options) {
    var result = []

    if (typeof options === 'number') {
        options = options.toString()
    }

    var matches = /^[0-9]+([,.-][0-9]+)?$/.exec(options)
    if (matches) {
        // split channel into discrete ircc commands
        for (var i = 0, len = options.length; i < len; i++) {
            if (options[i] === '.' || options[i] === '-') {
                result.push(KeyControl.Dash)
            } else {
                result.push('KeyControl.Num' + options[i])
            }
        }
    }

    return result
}

var removeCommandTimeout = function() {
    if (commandTimeout) {
        clearTimeout(commandTimeout);
        commandTimeout = null;
    }
}

var sendDeviceCommand = function(command, location) {
    commandVal = command;
    removeCommandTimeout();
    commandTimeout = setTimeout(function() {
        log("ERROR", 'CommandTimeout');
        try {
            sendCommandDetailSentEvent(false, commandVal, device.contextId, getSource(device.contextId));          
        } catch (error) {
            sendInsightEvent(false, commandVal); 			
        }
        disconnect();
    }, 2000);
    is_api_active();
    if (!isConnected) {
        wsConnect();
        return;
    }
    if (socket) {
        socket.send(JSON.stringify({
            'method': 'ms.remote.control',
            'params': {
                'DataOfCmd': commandMap[command],
                'Option': false,
                'TypeOfRemote': 'SendRemoteKey',
                'Cmd': 'Click'
            }
        }));
        if (isConnected) {
            try {
                sendCommandDetailSentEvent(true, command, device.contextId, getSource(device.contextId));
                log("DEBUG", "Sending ContextId from JS code: " + device.contextId);
            } catch (error) {
                sendInsightEvent(true, command);                    
				log("DEBUG", "Sending sendInsightEvent: true: " + command);
            }
        } else {
            try {
                sendCommandDetailSentEvent(false, command, device.contextId, getSource(device.contextId));
                log("DEBUG", "Sending ContextId from JS code: " + device.contextId);
            } catch (error) {
                sendInsightEvent(false, command);                    
				log("DEBUG", "Sending sendInsightEvent: true: " + command);
            }
        }
    } else {
        wsConnect();
    }
}

var success = function() {
    wsConnect();
}

var is_api_active = function(done) {
    try {
        fetch('http://' + device.ip + ':8001/api/v2/', {
            method: 'GET',
        }).then(function(response) {
            removeCommandTimeout();
            if (response.status === 200) {
                log('DEBUG', 'TV API is active');
            } else {
                disconnect();
                toggleAlertMessage("block");
                log('DEBUG', 'No response from TV');
            }
        }).catch(function(error) {
            log('DEBUG', 'No response from TV');
            toggleAlertMessage("none");
            disconnect();
            if (document.getElementById("loader-container")) {
                document.getElementById("loader-container").style.display = "none";
            }
            if (document.getElementById("tvPopOn")) {
                document.getElementById("tvPopOn").style.display = "none";
            }
            if (document.getElementById("pairingCodeBox")) {
                document.getElementById("pairingCodeBox").style.display = "none";
            }
        });
    } catch (e) {
        disconnect();
        log('DEBUG', 'Error: ' + e);
    }
};

var startTimer = function(event, command) {
    pressTimer = setTimeout(function() {
        sendSamsungCommand(event, command);
        startTimer(event, command);
    }, 500);
}

var clearTimer = function() {
    clearTimeout(pressTimer);
}


var sendSamsungCommand = function(event, command) {

    highlightingButtons(event, command);
    // wsConnect();

    setTimeout(function() {
        sendDeviceCommand(command, device.source);
    }, 100);
}

function initDeviceInfo() {
    try {
        device = JSON.parse(getInfo('currentDevice'));
        log("DEBUG", device.ip + " " + device.port);
        if (device.modelNumber) {
            if (device.friendlyName){
                document.getElementById("deviceInfo").innerHTML = `${device.modelNumber} - ${device.friendlyName}`;
            }
            else {
                document.getElementById("deviceInfo").innerHTML = `${device.brandName} - ${device.modelNumber}`;
            }
        }
        // wsConnect();
    } catch (err) {
        log("ERROR", "Unable to initialize device info: " + err);
    }
}

function getInfo(value) {
    var info = "empty";
    switch (value) {
        case "currentDevice":
            info = Android.getCurrentDeviceInfo();
            break;
        case "currentActivity":
            info = Android.getCurrentActivityInfo();
            break;
        case "currentRoom":
            info = Android.getCurrentRoomInfo();
            break;
    }
    return info;
}
