'use strict';

var device = {
    "ip": "192.168.50.90",
    "port": "1400",
    "contextId": 151,
    "source": "inapp"
};
var commandVal;
var commandStatus = '';
var commandTimeout;
var pressTimer;
var statusTimerCtrl = false;

var commands = {
    VolumeGet: {
        endpoint: '/MediaRenderer/RenderingControl/Control',
        schema: 'urn:schemas-upnp-org:service:RenderingControl:1',
        action: 'GetVolume'
    },

    VolumeSet: {
        endpoint: '/MediaRenderer/RenderingControl/Control',
        schema: 'urn:schemas-upnp-org:service:RenderingControl:1',
        action: 'SetVolume'
    },

    Volume_Up: {
        endpoint: '/MediaRenderer/RenderingControl/Control',
        schema: 'urn:schemas-upnp-org:service:RenderingControl:1',
        action: 'SetVolume'
    },

    Volume_Down: {
        endpoint: '/MediaRenderer/RenderingControl/Control',
        schema: 'urn:schemas-upnp-org:service:RenderingControl:1',
        action: 'SetVolume'
    },

    MuteGet: {
        endpoint: '/MediaRenderer/RenderingControl/Control',
        schema: 'urn:schemas-upnp-org:service:RenderingControl:1',
        action: 'GetMute'
    },

    Mute: {
        endpoint: '/MediaRenderer/RenderingControl/Control',
        schema: 'urn:schemas-upnp-org:service:RenderingControl:1',
        action: 'SetMute'
    },

    Play: {
        endpoint: '/MediaRenderer/AVTransport/Control',
        schema: 'urn:schemas-upnp-org:service:AVTransport:1',
        action: 'Play'
    },

    Pause: {
        endpoint: '/MediaRenderer/AVTransport/Control',
        schema: 'urn:schemas-upnp-org:service:AVTransport:1',
        action: 'Pause'
    },

    Previous: {
        endpoint: '/MediaRenderer/AVTransport/Control',
        schema: 'urn:schemas-upnp-org:service:AVTransport:1',
        action: 'Previous'
    },

    Next: {
        endpoint: '/MediaRenderer/AVTransport/Control',
        schema: 'urn:schemas-upnp-org:service:AVTransport:1',
        action: 'Next'
    },

    Stop: {
        endpoint: '/MediaRenderer/AVTransport/Control',
        schema: 'urn:schemas-upnp-org:service:AVTransport:1',
        action: 'Stop'
    }
}

var buildMessage = function (command, options) {
    var xmlStr = "<?xml version=\"1.0\" encoding=\"utf-8\"?>" +
        "<s:Envelope s:encodingStyle=\"http://schemas.xmlsoap.org/soap/encoding/\" xmlns:s=\"http://schemas.xmlsoap.org/soap/envelope/\">" +
        "<s:Body>" +
        "<u:" + command.action + " xmlns:u=\"" + command.schema + "\">" +
        "<InstanceID>0</InstanceID>" +
        "<Channel>Master</Channel>";
    if (command.action === 'Play') {
        xmlStr += "<Speed>1</Speed>"
    }
    if (options) {
        log("DEBUG", "Options : " + options);
        xmlStr += "<Desired" + command.action.substring(3) + ">" + options + "</Desired" + command.action.substring(3) + ">";
    }
    xmlStr += "</u:" + command.action + ">" +
        "</s:Body>" +
        "</s:Envelope>";
    console.log("DEBUG", "xml_str: " + xmlStr);
    log("DEBUG", "xml_str: " + xmlStr);
    return xmlStr;
}

var removeCommandTimeout = function () {
    if (commandTimeout) {
        toggleAlertMessage("none");
        clearTimeout(commandTimeout);
        commandTimeout = null;
    }
}

var sendRawCommand = function (command, options) {
    var soapbits = commands[command];
    var soapdata = buildMessage(soapbits, options);
    var url = `http://${device.ip}:${device.port}${soapbits.endpoint}`;
    var method = 'POST';
    var headersJsonMap = JSON.stringify({
        'SOAPACTION': soapbits.schema + '#' + soapbits.action,
        'Content-type': 'text/xml; charset=utf8'
    });
    removeCommandTimeout();
    commandTimeout = setTimeout(function () {
        toggleAlertMessage("block");
    }, 2000);
    console.log(soapdata, headersJsonMap, soapbits.endpoint);
    Android.sendHttpRequest(method, headersJsonMap, soapdata, url);

    try {
        if(device.contextId === 0){
             sendRemoteButtonPressed(141, "Sonos", command, getSource(device.contextId));
             log("DEBUG", "Collapsed Widget: " + device.contextId);
        }
        else{
        sendRemoteButtonPressed(device.contextId, "Sonos", command, getSource(device.contextId));
          log("DEBUG", "Expanded Widget: " + device.contextId);
          }
    } catch (error) {
        log("ERROR", "sendRemoteButtonPressed is not defined");
    }

}

var startTimer = function (event, command) {
    console.log("startTimer:" + command);
    pressTimer = setTimeout(function () {
        sendSonosCommand(event, command);
        startTimer(event, command);
    }, 100);
};

var clearTimer = function () {
    console.log("clearTimer:");
    clearTimeout(pressTimer);
}


function statusVolume() {
    try {
        if (statusTimerCtrl === true) {
            log("DEBUG", "contextId for status volume: " + device.contextId);
            commandStatus = "volumeStatus";
            sendRawCommand('VolumeGet');
        }
    } catch (error) { }
}



var sendSonosCommand = function (event, command) {
    highlightingButtons(event, command);
    sendDeviceCommand(command, "inApp");
   
}

var getHttpResponse = function (response) {
    removeCommandTimeout();
    response = response.split('|');
    var url = response[0];
    var status = response[1];
    var result = response[2];
    log("DEBUG", "result: " + result);
    if (parseInt(status) === 200) {
        let setVolumeVal;
        let muteVal;
        log("DEBUG", "commandStatus: " + commandStatus);
        switch (commandStatus) {
            case 'volumeUp':
                setVolumeVal = volumeUpValue(parseInt(/<CurrentVolume>(.*?)<\/CurrentVolume>/g.exec(result)[1], 10));
                commandStatus = '';
                log("DEBUG", "result: " + setVolumeVal);
                sendRawCommand('Volume_Up', setVolumeVal);
                //document.getElementById("statusVolume").innerHTML = "VOL: " + setVolumeVal;
                break;
            case 'volumeDown':
                setVolumeVal = volumeDownValue(parseInt(/<CurrentVolume>(.*?)<\/CurrentVolume>/g.exec(result)[1], 10));
                commandStatus = '';
                log("DEBUG", "result: " + setVolumeVal);
                sendRawCommand('Volume_Down', setVolumeVal);
                //document.getElementById("statusVolume").innerHTML = "VOL: " + setVolumeVal;
                break;
            case 'mute':
                log("DEBUG", "Reached mute val: ");
                var mute = /<CurrentMute>(.*?)<\/CurrentMute>/g.exec(result)[1]
                log("DEBUG", "var mute value : " + mute);
                if (mute === '0') {
                    log("DEBUG", "mute value 0: ");
                    mute = '1';
                } else {
                    log("DEBUG", "mute value 1: ");
                    mute = '0';
                }
                log("DEBUG", "Mute val: " + mute);
                commandStatus = '';
                sendRawCommand('Mute', mute);
                break;
            case 'volumeStatus':
                try {
                    commandStatus = '';
                    if (statusTimerCtrl === true) {
                        var currentVolumeVal = volumeDownValue(parseInt(/<CurrentVolume>(.*?)<\/CurrentVolume>/g.exec(result)[1], 10));
                        document.getElementById("statusVolume").innerHTML = "VOL: " + currentVolumeVal;
                    }
                } catch (errorstatus) {
                    console.log(errorstatus)
                }
                break;
            default:
                try {
                    sendCommandDetailSentEvent(true, commandVal, device.contextId, getSource(device.contextId));
                } catch (error) {
                    sendInsightEvent(true, commandVal);
                }
                break;
        }
    } else {
        try {
            sendCommandDetailSentEvent(false, commandVal, device.contextId, getSource(device.contextId));
        } catch (error) {
            sendInsightEvent(false, commandVal);
        }
        toggleAlertMessage("block");
        return;
    }
}

var volumeUpValue = function (volumeVal) {
    volumeVal += 2;
    log("DEBUG", "current Volume: " + volumeVal);
    if (volumeVal > 200) {
        volumeVal = 200
    }
    log("DEBUG", "volumeUpValue: " + volumeVal);
    try{document.getElementById("statusVolume").innerHTML = "VOL: " + volumeVal;
    }catch (errorstatus) {}
    return volumeVal;
}


var volumeDownValue = function (volumeVal) {
    volumeVal -= 2;
    log("DEBUG", "current Volume: " + volumeVal);
    if (volumeVal < 0) {
        volumeVal = 0
    }
    log("DEBUG", "volumeDownValue: " + volumeVal);
     try{document.getElementById("statusVolume").innerHTML = "VOL: " + volumeVal;
    }catch (errorstatus) {}
    return volumeVal;
}

var sendDeviceCommand = function (command, location) {
   
    statusTimerCtrl = true;
    statusVolume();
  

    commandVal = command;
    log("DEBUG", "commandVal: " + command);
    switch (command) {
        case 'Volume_Up':
            commandStatus = "volumeUp";
            log("DEBUG", "VolumeGet: " + command);
            sendRawCommand('VolumeGet');
            break;
        case 'Volume_Down':
            commandStatus = "volumeDown";
            log("DEBUG", "VolumeGet: " + command);
            sendRawCommand('VolumeGet');
            break;
        case 'Mute':
            commandStatus = "mute";
            log("DEBUG", "Mute: " + command);
            sendRawCommand('MuteGet');
            break;
        default:
            log("DEBUG", "default: " + command);
            sendRawCommand(command);
    }
}


function initDeviceInfo() {
    console.log("Init Device Info");
    try {
        device = JSON.parse(getInfo('currentDevice'));
        log("DEBUG", "initDeviceInfo: " + device);
        log("DEBUG", "getCurrentDeviceInfo(): " + JSON.stringify(device));
        if (device.modelNumber) {
            if (device.friendlyName) {
                document.getElementById("deviceInfo").innerHTML = `${device.friendlyName}`;
            }
            else {
                document.getElementById("deviceInfo").innerHTML = `${device.brandName} - ${device.modelNumber}`;
            }
        }
        //setInterval(statusVolume, 1000);
        statusTimerCtrl = true;
        statusVolume();
        
    } catch (err) {
        log("ERROR", "Unable to initialize device info: " + err);
    }
}

function getInfo(value) {
    var info = "empty";
    switch (value) {
        case "currentDevice":
            info = Android.getCurrentDeviceInfo();
            break;
        case "currentActivity":
            info = Android.getCurrentActivityInfo();
            break;
        case "currentRoom":
            info = Android.getCurrentRoomInfo();
            break;
    }
    return info;
}