'use strict';

var device = {
    "ip": "192.168.1.1",
    "port": "8080",	 
	"contextId":151, 
    "source":"inapp"
};
var commandsMap = {
    'Menu': 'Option',
    'Info': 'On Screen',
    'Navigate_Up': 'Up',
    'Navigate_Down': 'Down',
    'Navigate_Left': 'Left',
    'Navigate_Right': 'Right',
    'Select': 'Sel'
};
var commandVal;

var pressTimer;

var yamahaCommands;
var commandTimeout;

var removeCommandTimeout = function() {
    if (commandTimeout) {
        clearTimeout(commandTimeout);
        commandTimeout = null;
    }
}

var sendRawCommand = function(rawCommand) {
    console.log('command: ' + rawCommand);
    removeCommandTimeout();
    commandTimeout = setTimeout(function() {
        log("ERROR", 'commandTimeout');
        toggleAlertMessage('block');
    }, 2000);
    fetch(`http://${device.ip}/YamahaRemoteControl/ctrl`, {
        method: 'POST',
        body: rawCommand,
        mode: 'no-cors',
        headers: {
            'Content-Length': rawCommand.length
        }
    }).then(function(res) {
        console.log(res);
        removeCommandTimeout();
        toggleAlertMessage("none");
        try{
            sendCommandDetailSentEvent(true, commandVal, device.contextId, getSource(device.contextId));
        }catch(error){
            sendInsightEvent(true, commandVal);
        }

    }).catch(function(error) {
        console.log(error);
        toggleAlertMessage("block");
        try{
            sendCommandDetailSentEvent(false, commandVal, device.contextId, getSource(device.contextId));
        }catch(error){
            sendInsightEvent(false, commandVal);
        }
    });
}

var disconnect = function() {
    toggleAlertMessage("block");
}

var startTimer = function(event, command) {
    console.log("startTimer:" + command);
    pressTimer = setTimeout(function() {
        sendYamahaCommand(event, command);
        startTimer(event, command);
    }, 100);
}

var clearTimer = function() {
    console.log("clearTimer:");
    clearTimeout(pressTimer);
}

var sendYamahaCommand = function(event, command) {
    highlightingButtons(event, command);
    sendDeviceCommand(command, "inApp");
}

function sendDeviceCommand(command, location) {
    console.log("sendDeviceCommand :" + command);
    toggleAlertMessage("none");
    commandVal = command;
    var rawCommand;
    if (commandsMap[command]) {
        command = commandsMap[command];
    }

    if (!yamahaCommands) {
        console.log("set yamahaCommands");
        yamahaCommands = new YamahaCommands()
    }

    switch (command) {
        case 'Power':
            rawCommand = yamahaCommands.power();
            return;
        case 'Mute':
            rawCommand = yamahaCommands.mute();
            return;
            // rawCommand === 'Off' ? yamahaCommands.muteOn() : yamahaCommands.muteOff();
        case 'Volume_Up':
            rawCommand = yamahaCommands.volumeUp(5);
            return;
        case 'Volume_Down':
            rawCommand = yamahaCommands.volumeDown(5);
            return;
        case 'Rewind':
            rawCommand = yamahaCommands.rewind();
            break;
        case 'Fast_Forward':
            rawCommand = yamahaCommands.skip();
            break;
        case 'Stop':
            rawCommand = yamahaCommands.stop();
            break;
        case 'Play':
            rawCommand = yamahaCommands.play();
            break;
        case 'Pause':
            rawCommand = yamahaCommands.pause();
            break;
        case 'Up':
        case 'Down':
        case 'Left':
        case 'Right':
        case 'Sel':
            rawCommand = yamahaCommands.sendCursorCommand(command);
            break;
        case 'Option':
        case 'On Screen':
            rawCommand = yamahaCommands.sendMenuCommand(command);
            break;
        default:
            rawCommand = yamahaCommands.rawCommand(command);
            break;
    }

    sendRawCommand(rawCommand);
}

function initDeviceInfo() {
    console.log("Init Device Info");
    try {
        device = JSON.parse(getInfo('currentDevice'));
        log("DEBUG", JSON.stringify(device));
        if (device.modelNumber) {
            if (device.friendlyName){
                document.getElementById("deviceInfo").innerHTML = `${device.modelNumber} - ${device.friendlyName}`;
            }
            else {
                document.getElementById("deviceInfo").innerHTML = `${device.brandName} - ${device.modelNumber}`;
            }
        }
        yamahaCommands = new YamahaCommands();
        // checkIsPairingDone(device.ip);
    } catch (err) {
        log("ERROR", "Unable to initialize device info: " + err);
    }
}


function getInfo(value) {
    var info = "empty";
    switch (value) {
        case "currentDevice":
            info = Android.getCurrentDeviceInfo();
            break;
        case "currentActivity":
            info = Android.getCurrentActivityInfo();
            break;
        case "currentRoom":
            info = Android.getCurrentRoomInfo();
            break;
    }
    return info;
}