'use strict';

var commandStatus = '';

function YamahaCommands() {}

function getZone(zone) {
    if (!zone) return "Main_Zone";

    // replace numbers with zones, eg type "2" and it will change to Zone 2
    if (zone.length == 1) {
        zone = zone.replace("/^1", "Main_Zone");
        zone = zone.replace("/^2", "Zone_2");
        zone = zone.replace("/^3", "Zone_3");
        zone = zone.replace("/^4", "Zone_4");
    }

    switch (zone) {
        case 1:
            zone = "Main_Zone";
            break;
        case 2:
        case 3:
        case 4:
            zone = "Zone_" + zone;
    }
    return zone;
}

YamahaCommands.prototype.mute = function(zone) {
    commandStatus = 'mute'
    var self = this;
    return self.getBasicInfo(zone);
}

YamahaCommands.prototype.power = function(zone) {
    commandStatus = 'power';
    var self = this;
    self.getBasicInfo(zone);
}

YamahaCommands.prototype.muteOn = function(zone) {
    var command = '<YAMAHA_AV cmd="PUT"><' + getZone(zone) + '><Volume><Mute>On</Mute></Volume></' + getZone(zone) + '></YAMAHA_AV>';
    return command;
};

YamahaCommands.prototype.muteOff = function(zone) {
    var command = '<YAMAHA_AV cmd="PUT"><' + getZone(zone) + '><Volume><Mute>Off</Mute></Volume></' + getZone(zone) + '></YAMAHA_AV>';
    return command;
};

YamahaCommands.prototype.stop = function(zone) {
    var command = '<YAMAHA_AV cmd="PUT"><' + getZone(zone) + '><Play_Control><Playback>Stop</Playback></Play_Control></' + getZone(zone) + '></YAMAHA_AV>';
    return command;
};

YamahaCommands.prototype.pause = function(zone) {
    var command = '<YAMAHA_AV cmd="PUT"><' + getZone(zone) + '><Play_Control><Playback>Pause</Playback></Play_Control></' + getZone(zone) + '></YAMAHA_AV>';
    return command;
};

YamahaCommands.prototype.play = function(zone) {
    var command = '<YAMAHA_AV cmd="PUT"><' + getZone(zone) + '><Play_Control><Playback>Play</Playback></Play_Control></' + getZone(zone) + '></YAMAHA_AV>';
    return command;
};

YamahaCommands.prototype.skip = function(zone) {
    var command = '<YAMAHA_AV cmd="PUT"><' + getZone(zone) + '><Play_Control><Playback>Skip Fwd</Playback></Play_Control></' + getZone(zone) + '></YAMAHA_AV>';
    return command;
};

YamahaCommands.prototype.rewind = function(zone) {
    var command = '<YAMAHA_AV cmd="PUT"><' + getZone(zone) + '><Play_Control><Playback>Skip Rev</Playback></Play_Control></' + getZone(zone) + '></YAMAHA_AV>';
    return command;
};

YamahaCommands.prototype.powerOn = function(zone) {
    var command = '<YAMAHA_AV cmd="PUT"><' + getZone(zone) + '><Power_Control><Power>On</Power></Power_Control></' + getZone(zone) + '></YAMAHA_AV>'
    return command;
};

YamahaCommands.prototype.powerOff = function(zone) {
    var command = '<YAMAHA_AV cmd="PUT"><' + getZone(zone) + '><Power_Control><Power>Standby</Power></Power_Control></' + getZone(zone) + '></YAMAHA_AV>';
    return command;
};

var setVolumeTo = function(to, zone) {
    var command = '<YAMAHA_AV cmd="PUT"><' + getZone(zone) + '><Volume><Lvl><Val>' + to + '</Val><Exp>1</Exp><Unit>dB</Unit></Lvl></Volume></' + getZone(zone) + '></YAMAHA_AV>';
    return command;
};
YamahaCommands.prototype.setVolume = YamahaCommands.prototype.setVolumeTo;

YamahaCommands.prototype.volumeUp = function(by, zone) {
    commandStatus = 'volumeUp';
    this.adjustVolumeBy(by, zone);
};

YamahaCommands.prototype.volumeDown = function(by, zone) {
    commandStatus = 'volumeDown';
    this.adjustVolumeBy(-by, zone);
};

YamahaCommands.prototype.adjustVolumeBy = function(by, zone) {
    if (typeof by == 'string' || by instanceof String) by = parseInt(by);
    var self = this;
    // self.getBasicInfo(zone);
    self.getBasicInfo(zone);
};

YamahaCommands.prototype.setMainInputTo = function(to) {
    return this.setInputTo(to, "Main_Zone");
};

YamahaCommands.prototype.setInputTo = function(to, zone) {
    var command = '<YAMAHA_AV cmd="PUT"><' + getZone(zone) + '><Input><Input_Sel>' + to + '</Input_Sel></Input></' + getZone(zone) + '></YAMAHA_AV>';
    return command;
};

YamahaCommands.prototype.setPureDirect = function(on) {
    return this.sendPutCommand('<Sound_Video><Pure_Direct><Mode>' + (on ? 'On' : 'Off') + '</Mode></Pure_Direct></Sound_Video>');
};
YamahaCommands.prototype.setHDMIOutput = function(hdmi_num, on) {
    return this.sendPutCommand('<Sound_Video><HDMI><Output><OUT_' + hdmi_num + '>' + (on ? 'On' : 'Off') + '</OUT_' + hdmi_num + '></Output></HDMI></Sound_Video>', 'System');
};

YamahaCommands.prototype.sleep = function(val, zone) {
    if (val < 30) val = 'Off';
    else if (val < 60) val = '30 min';
    else if (val < 90) val = '60 min';
    else if (val < 120) val = '90 min';
    else val = '120 min';
    return this.sendPutCommand('<Power_Control><Sleep>' + val + '</Sleep></Power_Control>', zone);
};

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


YamahaCommands.prototype.setBassTo = function(to) {
    var zone = getZone(); //only available in Main Zone
    var command = '<YAMAHA_AV cmd="PUT"><' + zone + '><Sound_Video><Tone><Bass><Val>' + to + '</Val><Exp>1</Exp><Unit>dB</Unit></Bass></Tone></Sound_Video></' + zone + '></YAMAHA_AV>';
    return command;
};

YamahaCommands.prototype.sendPutCommand = function(command, zone) {
    zone = getZone(zone);
    command = '<YAMAHA_AV cmd="PUT"><' + zone + '>' + command + '</' + zone + '></YAMAHA_AV>';
    return command;
};

YamahaCommands.prototype.sendCursorCommand = function(command, zone) {
    zone = getZone(zone);
    command = '<YAMAHA_AV cmd="PUT"><Main_Zone><Cursor_Control><Cursor>' + command + '</Cursor></Cursor_Control></Main_Zone></YAMAHA_AV>';
    return command;
}

YamahaCommands.prototype.sendMenuCommand = function(command, zone) {
    zone = getZone(zone);
    command = '<YAMAHA_AV cmd="PUT"><Main_Zone><Cursor_Control><Menu_Control>' + command + '</Menu_Control></Cursor_Control></Main_Zone></YAMAHA_AV>';
    return command;
}

YamahaCommands.prototype.rawCommand = function(command, zone) {
    zone = getZone(zone);
    command = '<YAMAHA_AV cmd="PUT"><Main_Zone><Input><Input_Sel>' + command + '</Input_Sel></Input></Main_Zone></YAMAHA_AV>';
    return command;
}

function xmlToJson(xml) {

    // Create the return object
    var obj = {};

    if (xml.nodeType == 1) { // element
        // do attributes
        if (xml.attributes.length > 0) {
            obj["attributes"] = {};
            for (var j = 0; j < xml.attributes.length; j++) {
                var attribute = xml.attributes.item(j);
                obj["attributes"][attribute.nodeName] = attribute.nodeValue;
            }
        }
    } else if (xml.nodeType == 3) { // text
        obj = xml.nodeValue;
    }

    // do children
    // If just one text node inside
    if (xml.hasChildNodes() && xml.childNodes.length === 1 && xml.childNodes[0].nodeType === 3) {
        obj = xml.childNodes[0].nodeValue;
    } else if (xml.hasChildNodes()) {
        for (var i = 0; i < xml.childNodes.length; i++) {
            var item = xml.childNodes.item(i);
            var nodeName = item.nodeName;
            if (typeof(obj[nodeName]) == "undefined") {
                obj[nodeName] = xmlToJson(item);
            } else {
                if (typeof(obj[nodeName].push) == "undefined") {
                    var old = obj[nodeName];
                    obj[nodeName] = [];
                    obj[nodeName].push(old);
                }
                obj[nodeName].push(xmlToJson(item));
            }
        }
    }
    return obj;
}

YamahaCommands.prototype.getBasicInfo = function(zone) {
    var self = this;
    var command = '<YAMAHA_AV cmd="GET"><' + getZone(zone) + '><Basic_Status>GetParam</Basic_Status></' + getZone(zone) + '></YAMAHA_AV>';
    self.sendRequest(command);
};

YamahaCommands.prototype.sendRequest = function(command) {
    var method = 'POST';
    var headersJsonMap = JSON.stringify({
        'Content-Type': 'text/xml'
    });
    var url = `http://${device.ip}/YamahaRemoteControl/ctrl`;
    var payload = command;
    removeCommandTimeout();
    commandTimeout = setTimeout(function() {
        log("ERROR", 'commandTimeout');
        toggleAlertMessage('block');

    }, 2000);
    Android.sendHttpRequest(method, headersJsonMap, payload, url);
}

function getHttpResponse(response) {
    removeCommandTimeout();
    var commandToSend;
    response = response.split('|');
    var url = response[0];
    var status = response[1];
    var result = response[2];

    if (parseInt(status) === 200) {
        toggleAlertMessage("none");
        result = new DOMParser().parseFromString(result, 'text/xml');
        result = xmlToJson(result);
        console.log(JSON.stringify(result));
        var avrStatus = enrichBasicStatus(result);
        switch (commandStatus) {
            case 'mute':
                console.log(avrStatus.isMuted());
                commandToSend = avrStatus.isMuted() === 'Off' ? yamahaCommands.muteOn() : yamahaCommands.muteOff();
                break;
            case 'power':
                console.log(avrStatus.isOn());
                commandToSend = avrStatus.isOn() === 'On' ? yamahaCommands.powerOff() : yamahaCommands.powerOn();
                break;
            case 'volumeUp':
                console.log(avrStatus.getVolume());
                var volume = avrStatus.getVolume() + 5;
                commandToSend = setVolumeTo(volume);
                break;
            case 'volumeDown':
                console.log(avrStatus.getVolume());
                var volume = avrStatus.getVolume() - 5;
                commandToSend = setVolumeTo(volume);
                break;
        }
        if (commandToSend) {
            sendRawCommand(commandToSend);
        }
    } else {
        sendInsightEvent(false, commandVal);
        toggleAlertMessage("block");
        return;
    }

}

// function getHttpJsonResponse(response) {
//     log("DEBUG", response);
//     response = JSON.parse(response);
//     log("DEBUG", "parsed and stringified: " + JSON.stringify(response));
//     var result = xmlToJson(response.resut);
//     console.log(result);
// }

function enrichBasicStatus(basicStatus, zone) {
    zone = getZone(zone);

    basicStatus.getVolume = function() {
        return parseInt(basicStatus.YAMAHA_AV[zone].Basic_Status.Volume.Lvl.Val);
    };

    basicStatus.isMuted = function() {
        return basicStatus.YAMAHA_AV[zone].Basic_Status.Volume.Mute;
    };

    basicStatus.isOn = function() {
        return basicStatus.YAMAHA_AV[zone].Basic_Status.Power_Control.Power;
    };

    basicStatus.isOff = function() {
        return !basicStatus.isOn();
    };

    basicStatus.getCurrentInput = function() {
        return basicStatus.YAMAHA_AV[zone].Basic_Status.Input.Input_Sel;
    };
    return basicStatus;
}


// Add direct functions for basic info
function addBasicInfoWrapper(basicInfo) {
    YamahaCommands.prototype[basicInfo] = function(zone) {
        return this.getBasicInfo(zone).then(function(result) {
            return result[basicInfo]();
        });
    };
}
//TODO: no list, take properties of basicStatus object
var basicInfos = ["getVolume", "isMuted", "isOn", "isOff", "getCurrentInput", "isPartyModeEnabled", "isPureDirectEnabled", "getBass", "getTreble", "getSubwooferTrim", "getDialogueLift", "getDialogueLevel", "isYPAOVolumeEnabled", "isExtraBassEnabled", "isAdaptiveDRCEnabled"];
for (var i = 0; i < basicInfos.length; i++) {
    var basicInfo = basicInfos[i];
    addBasicInfoWrapper(basicInfo);
}

YamahaCommands.prototype.getSystemConfig = function() {
    var command = '<YAMAHA_AV cmd="GET"><System><Config>GetParam</Config></System></YAMAHA_AV>';
    return this.SendXMLToReceiver(command).then(xml2js.parseStringAsync);
};

YamahaCommands.prototype.getZoneConfig = function(zone) {
    var command = '<YAMAHA_AV cmd="GET"><' + getZone(zone) + '><Config>GetParam</Config></' + getZone(zone) + '></YAMAHA_AV>';
    return this.SendXMLToReceiver(command).then(xml2js.parseStringAsync);
};

YamahaCommands.prototype.getAvailableZones = function() {
    return this.getSystemConfig().then(function(info) {
        var zones = [];
        var zonesXML = info.YAMAHA_AV.System[0].Config[0].Feature_Existence[0];
        for (var prop in zonesXML) {
            // Only return zones that the receiver supports
            if (prop.includes('one') && zonesXML[prop].includes('1')) {
                zones.push(prop);
            }
        }
        return zones;
    });
};

YamahaCommands.prototype.getAvailableInputs = function() {
    return this.getSystemConfig().then(function(info) {
        var inputs = [];
        var inputsXML = info.YAMAHA_AV.System[0].Config[0].Name[0].Input[0];
        for (var prop in inputsXML) {
            inputs.push(inputsXML[prop][0]);
        }
        return inputs;
    });
};

YamahaCommands.prototype.selectListItem = function(listname, number) {
    var command = '<YAMAHA_AV cmd="PUT"><' + listname + '><List_Control><Direct_Sel>Line_' + number + '</Direct_Sel></List_Control></' + listname + '></YAMAHA_AV>';
    return this.SendXMLToReceiver(command);
};

YamahaCommands.prototype.getList = function(name) {
    var command = '<YAMAHA_AV cmd="GET"><' + name + '><List_Info>GetParam</List_Info></' + name + '></YAMAHA_AV>';
    return this.SendXMLToReceiver(command).then(xml2js.parseStringAsync).then(function(info) {
        enrichListInfo(info, name);
        return info;
    });
};

function enrichListInfo(listInfo, listname) {

    listInfo.hasSelectableItems = function() {
        return listInfo.YAMAHA_AV[listname][0].List_Info[0].Current_List[0].Line_1[0].Attribute[0] !== "Unselectable";
    };

    listInfo.isReady = function() {

        return !listInfo.isBusy() && listInfo.hasSelectableItems();
    };

    listInfo.isBusy = function() {
        return listInfo.YAMAHA_AV[listname][0].List_Info[0].Menu_Status[0] === "Busy";
    };

    listInfo.getMenuLayer = function() {
        return listInfo.YAMAHA_AV[listname][0].List_Info[0].Menu_Layer[0];
    };

    listInfo.getMenuName = function() {
        return listInfo.YAMAHA_AV[listname][0].List_Info[0].Menu_Name[0];
    };

}


YamahaCommands.prototype.isMenuReady = function(name) {
    var self = this;
    return self.getList(name).then(function(result) {
        return result.isReady();
    });
};

YamahaCommands.prototype.whenMenuReady = function(name) {
    var self = this;
    return self.when("isMenuReady", name, true);
};

YamahaCommands.prototype.when = function(YamahaCall, parameter, expectedReturnValue, tries) {
    var self = this;
    tries = tries || 0;
    return this[YamahaCall](parameter).then(function(result) {
        console.log('Polling...');
        if (result == expectedReturnValue) {
            return true;
        } else if (tries > 40) {
            return Promise.reject("Timeout");
        } else {
            return Promise.delay(self.pollingDelay).then(function() {
                return self.when(YamahaCall, parameter, expectedReturnValue, tries + 1);
            });
        }
    });
};

YamahaCommands.prototype.selectUSBListItem = function(number) {
    return this.selectListItem("USB", number);
};

YamahaCommands.prototype.selectWebRadioListItem = function(number) {
    return this.selectListItem("NET_RADIO", number);
};

YamahaCommands.prototype.selectTunerPreset = function(number) {
    var command = '<YAMAHA_AV cmd="PUT"><Tuner><Play_Control><Preset><Preset_Sel>' + number + '</Preset_Sel></Preset></Play_Control></Tuner></YAMAHA_AV>';
    return this.SendXMLToReceiver(command);
};

YamahaCommands.prototype.getTunerPresetList = function() {
    var command = '<YAMAHA_AV cmd="GET"><Tuner><Play_Control><Preset><Data>GetParam</Data></Preset></Play_Control></Tuner></YAMAHA_AV>';

    return this.SendXMLToReceiver(command).then(xml2js.parseStringAsync).then(function(info) {
        var presets = {};
        for (var presetNum in info.YAMAHA_AV.Tuner[0].Play_Control[0].Preset[0].Data[0]) {
            var preset = info.YAMAHA_AV.Tuner[0].Play_Control[0].Preset[0].Data[0][presetNum][0];
            if (preset.Status[0] === 'Exist') {
                var number = presetNum.substring(7);
                var band = preset.Band[0];
                if (band == 'AM') {
                    var freq = preset.Freq[0].AM[0].Val[0] / Math.pow(10, preset.Freq[0].AM[0].Exp[0]);
                } else {
                    var freq = preset.Freq[0].FM[0].Val[0] / Math.pow(10, preset.Freq[0].FM[0].Exp[0]);
                }
                presets[number] = {
                    "preset": number,
                    "band": band,
                    "value": freq
                }
            }
        }
        return (presets);
    });
};

YamahaCommands.prototype.getTunerInfo = function() {
    var command = '<YAMAHA_AV cmd="GET"><Tuner><Play_Info>GetParam</Play_Info></Tuner></YAMAHA_AV>';

    return this.SendXMLToReceiver(command).then(xml2js.parseStringAsync).then(function(info) {
        var presets = info.YAMAHA_AV.Tuner[0];
        return (presets);
    });
};

YamahaCommands.prototype.selectTunerFrequency = function(band, frequency) {
    var unit = band == "FM" ? "MHz" : "KHz",
        exp = band == "FM" ? 2 : 0,
        command = '<YAMAHA_AV cmd="PUT"><Tuner><Play_Control><Tuning><Band>' + band + '</Band><Freq><' + band + '><Val>' + frequency + '</Val><Exp>' + exp + '</Exp><Unit>' + unit + '</Unit></' + band + '></Freq></Tuning></Play_Control></Tuner></YAMAHA_AV>';
    return this.SendXMLToReceiver(command);
};

//TODO: More XML CONVERT
YamahaCommands.prototype.getWebRadioList = function() {
    return this.getList("NET_RADIO");
};
YamahaCommands.prototype.getUSBList = function() {
    return this.getList("USB");
};